searchData={"content_type":"text/markdown","items":[{"doc":"The `ExAliyunOts` module provides a tablestore-based API as a client for working with Alibaba TableStore product servers.\n\nHere are links to official documents in [Chinese](https://help.aliyun.com/document_detail/27280.html) | [English](https://www.alibabacloud.com/help/product/27278.html)","ref":"ExAliyunOts.html","title":"ExAliyunOts","type":"module"},{"doc":"config :ex_aliyun_ots, :my_instance\n      name: \"MyInstanceName\",\n      endpoint: \"MyInstanceEndpoint\",\n      access_key_id: \"MyAliyunRAMKeyID\",\n      access_key_secret: \"MyAliyunRAMKeySecret\"\n\n    config :ex_aliyun_ots,\n      instances: [:my_instance],\n      debug: false,\n      enable_tunnel: false\n\n* `debug`, optional, specifies whether to enable debug logger, by default it's false, and please DO NOT use debug mode in production.\n* `enable_tunnel`, optional, specifies whether to enable tunnel functions, there will startup tunnel related `Supervisor` and `Registry` when enable it, by default it's false.","ref":"ExAliyunOts.html#module-configuration","title":"Configuration - ExAliyunOts","type":"module"},{"doc":"To use `ExAliyunOts`, a module that calls `use ExAliyunOts` has to be defined:\n\n    defmodule MyApp.TableStore do\n      use ExAliyunOts, instance: :my_instance\n    end\n\nThis automatically defines some macros and functions in the `MyApp.TableStore` module, here are some examples:\n\n    import MyApp.TableStore\n\n    # Create table\n    create_table \"table\",\n      [{\"pk1\", :integer}, {\"pk2\", :string}]\n\n    # Put row\n    put_row \"table\",\n      [{\"pk1\", \"id1\"}],\n      [{\"attr1\", 10}, {\"attr2\", \"attr2_value\"}],\n      condition: :expect_not_exist,\n      return_type: :pk\n\n    # Search index\n    search \"table\", \"index_name\",\n      search_query: [\n        query: match_query(\"age\", 28),\n        sort: [\n          field_sort(\"age\", order: :desc)\n        ]\n      ]\n\n    # Local transaction\n    start_local_transaction \"table\", {\"partition_key\", \"partition_value\"}","ref":"ExAliyunOts.html#module-using-exaliyunots","title":"Using ExAliyunOts - ExAliyunOts","type":"module"},{"doc":"There are two ways to use ExAliyunOts:\n\n  * using macros and functions from your own ExAliyunOts module, like `MyApp.TableStore`.\n  * using macros and functions from the `ExAliyunOts` module.\n\nAll defined functions and macros in `ExAliyunOts` are available and referable for your own ExAliyunOts module as well, except that the given arity of functions may\ndifferent, because the `instance` parameter of each invoke request is NOT needed from your own ExAliyunOts module although the `ExAliyunOts` module defines it.","ref":"ExAliyunOts.html#module-exaliyunots-api","title":"ExAliyunOts API - ExAliyunOts","type":"module"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/93819.html) | [English](https://www.alibabacloud.com/help/doc-detail/93819.html)","ref":"ExAliyunOts.html#abort_transaction/2","title":"ExAliyunOts.abort_transaction/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    abort_transaction(\"transaction_id\")","ref":"ExAliyunOts.html#abort_transaction/2-example","title":"Example - ExAliyunOts.abort_transaction/2","type":"function"},{"doc":"","ref":"ExAliyunOts.html#add_defined_columns/3","title":"ExAliyunOts.add_defined_columns/3","type":"function"},{"doc":"import MyApp.TableStore\n\n    columns = [\n        {\"attr1\", :string},\n        {\"attr2\", :integer},\n        {\"attr3\", :boolean},\n        {\"attr4\", :double},\n        {\"attr5\", :binary}\n      ]\n\n    add_defined_columns(\"table\", columns)","ref":"ExAliyunOts.html#add_defined_columns/3-example","title":"Example - ExAliyunOts.add_defined_columns/3","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/437170.html) | [English](https://www.alibabacloud.com/help/doc-detail/437170.html)","ref":"ExAliyunOts.html#alter_table_add_column/4","title":"ExAliyunOts.alter_table_add_column/4","type":"function"},{"doc":"import MyApp.TableStore\n\n    alter_table_add_column(\"table\", \"column\", \"BOOL\")","ref":"ExAliyunOts.html#alter_table_add_column/4-example","title":"Example - ExAliyunOts.alter_table_add_column/4","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/437170.html) | [English](https://www.alibabacloud.com/help/doc-detail/437170.html)","ref":"ExAliyunOts.html#alter_table_drop_column/3","title":"ExAliyunOts.alter_table_drop_column/3","type":"function"},{"doc":"import MyApp.TableStore\n\n    alter_table_drop_column(\"table\", \"column\")","ref":"ExAliyunOts.html#alter_table_drop_column/3-example","title":"Example - ExAliyunOts.alter_table_drop_column/3","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/27310.html) | [English](https://www.alibabacloud.com/help/doc-detail/27310.html)","ref":"ExAliyunOts.html#batch_get/2","title":"ExAliyunOts.batch_get/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    batch_get [\n      get(table_name1, [[{\"key1\", 1}, {\"key2\", \"1\"}]]),\n      get(\n        table_name2,\n        [{\"key1\", \"key1\"}],\n        columns_to_get: [\"name\", \"age\"],\n        filter: filter \"age\" >= 10\n      )\n    ]\n\nThe batch get operation can be considered as a collection of mulitple `get/3` operations.","ref":"ExAliyunOts.html#batch_get/2-example","title":"Example - ExAliyunOts.batch_get/2","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/27311.html) | [English](https://www.alibabacloud.com/help/doc-detail/27311.html)","ref":"ExAliyunOts.html#batch_write/3","title":"ExAliyunOts.batch_write/3","type":"function"},{"doc":"import MyApp.TableStore\n\n    batch_write [\n      {\"table1\", [\n        write_delete([{\"key1\", 5}, {\"key2\", \"5\"}],\n          return_type: :pk,\n          condition: condition(:expect_exist, \"attr1\" == 5)),\n        write_put([{\"key1\", 6}, {\"key2\", \"6\"}],\n          [{\"new_put_val1\", \"val1\"}, {\"new_put_val2\", \"val2\"}],\n          condition: condition(:expect_not_exist),\n          return_type: :pk)\n      ]},\n      {\"table2\", [\n        write_update([{\"key1\", \"new_tab3_id2\"}],\n          put: [{\"new_put1\", \"u1\"}, {\"new_put2\", 2.5}],\n          condition: :expect_not_exist),\n        write_put([{\"key1\", \"new_tab3_id3\"}],\n          [{\"new_put1\", \"put1\"}, {\"new_put2\", 10}],\n          condition: :expect_not_exist)\n      ]}\n    ]\n\nThe batch write operation can be considered as a collection of multiple `write_put/3`, `write_update/2` and `write_delete/2` operations.","ref":"ExAliyunOts.html#batch_write/3-example","title":"Example - ExAliyunOts.batch_write/3","type":"function"},{"doc":"* `:transaction_id`, optional, batch write operation within local transaction.\n  * `:is_atomic`, optional, defaults to false, whether set this batch write request be with an atomic operation, if this option is `true`,\n    keep the partition key of each table in the batch write operation is unique, or the corresponding write operation of the table will fail.","ref":"ExAliyunOts.html#batch_write/3-options","title":"Options - ExAliyunOts.batch_write/3","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/93819.html) | [English](https://www.alibabacloud.com/help/doc-detail/93819.html)","ref":"ExAliyunOts.html#commit_transaction/2","title":"ExAliyunOts.commit_transaction/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    commit_transaction(\"transaction_id\")","ref":"ExAliyunOts.html#commit_transaction/2-example","title":"Example - ExAliyunOts.commit_transaction/2","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/53813.html) | [English](https://www.alibabacloud.com/help/doc-detail/53813.html)","ref":"ExAliyunOts.html#compute_split_points_by_size/3","title":"ExAliyunOts.compute_split_points_by_size/3","type":"function"},{"doc":"Query current supported maximum number of concurrent tasks to `parallel_scan/4` request.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/153862.html) | [English](https://www.alibabacloud.com/help/doc-detail/153862.htm)","ref":"ExAliyunOts.html#compute_splits/3","title":"ExAliyunOts.compute_splits/3","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/35194.html) | [English](https://www.alibabacloud.com/help/doc-detail/35194.html)","ref":"ExAliyunOts.html#condition/1","title":"ExAliyunOts.condition/1","type":"macro"},{"doc":"import MyApp.TableStore\n\n    update_row \"table\", [{\"pk\", \"pk1\"}],\n      delete_all: [\"attr1\", \"attr2\"],\n      return_type: :pk,\n      condition: condition(:expect_exist)\n\nThe available `existence` options: `:expect_exist` | `:expect_not_exist` | `:ignore`, here are some use cases for your reference:\n\nUse `condition(:expect_exist)`, expect the primary keys to row is existed.\n  * for `put_row/5`, if the primary keys have auto increment column type, meanwhile the target primary keys row is existed,\n  only use `condition(:expect_exist)` can successfully overwrite the row.\n  * for `update_row/4`, if the primary keys have auto increment column type, meanwhile the target primary keys row is existed,\n  only use `condition(:expect_exist)` can successfully update the row.\n  * for `delete_row/4`, no matter what primary keys type are, use `condition(:expect_exist)` can successfully delete the row.\n\nUse `condition(:expect_not_exist)`, expect the primary_keys to row is not existed.\n  * for `put_row/5`, if the primary keys have auto increment type,\n    - while the target primary keys row is existed, only use `condition(:expect_exist)` can successfully put the row;\n    - while the target primary keys row is not existed, only use `condition(:ignore)` can successfully put the row.\n\nUse `condition(:ignore)`, ignore the row existence check\n  * for `put_row/5`, if the primary keys have auto increment column type, meanwhile the target primary keys row is not existed,\n  only use `condition(:ignore)` can successfully put the row.\n  * for `update_row/4`, if the primary keys have auto increment column type, meanwhile the target primary keys row is not existed,\n  only use `condition(:ignore)` can successfully update the row.\n  * for `delete_row/4`, no matter what primary keys type are, use `condition(:ignore)` can successfully delete the row if existed.\n\nThe `batch_write/3` operation is a collection of put_row / update_row / delete_row operations.","ref":"ExAliyunOts.html#condition/1-example","title":"Example - ExAliyunOts.condition/1","type":"macro"},{"doc":"Similar to `condition/1` and support use filter expression (please see `filter/1`) as well, please refer them for details.","ref":"ExAliyunOts.html#condition/2","title":"ExAliyunOts.condition/2","type":"macro"},{"doc":"import MyApp.TableStore\n\n    delete_row \"table\",\n      [{\"key\", \"key1\"}, {\"key2\", \"key2\"}],\n      condition: condition(:expect_exist, \"attr_column\" == \"value2\")","ref":"ExAliyunOts.html#condition/2-example","title":"Example - ExAliyunOts.condition/2","type":"macro"},{"doc":"Create secondary indexes. Official document in [Chinese](https://help.aliyun.com/document_detail/91947.html) | [English](https://www.alibabacloud.com/help/doc-detail/91947.html)","ref":"ExAliyunOts.html#create_index/6","title":"ExAliyunOts.create_index/6","type":"function"},{"doc":"# global secondary index\n    create_index \"table_name\",\n      \"table_index_global\",\n      [\"col0\", \"pk2\"],\n      [\"col1\", \"col2\"]\n\n    # local secondary index\n    create_index \"table_name\",\n      \"table_index_local\",\n      [\"pk0\", \"col1\"],\n      [\"col2\", \"col3\"],\n      index_type: :local\n\n    # do NOT include existing data in base table\n    create_index \"table_name\",\n      \"table_index_global_without_existing_data\",\n      [\"col0\", \"pk1\"],\n      [\"col1\", \"col2\", \"col3\"],\n      include_base_data: false","ref":"ExAliyunOts.html#create_index/6-example","title":"Example - ExAliyunOts.create_index/6","type":"function"},{"doc":"* `:index_type`, the type of the index, optional. Valid values: `:global` and `:local`. By default it is `:global`.\n    - If `:index_type` is not specified or is set to `:global`, the global secondary index feature is used.\n    If you use the global secondary index feature, Tablestore automatically synchronizes the columns to be indexed and data in primary key columns from a data table to an index table in asynchronous mode.\n    The synchronization latency is within a few milliseconds.\n    - If `:index_type` is set to `:local`, the local secondary index feature is used.\n    If you use the local secondary index feature, Tablestore automatically synchronizes data from the indexed columns and the primary key columns of a data table to the columns of an index table in synchronous mode.\n    After the data is written to the data table, you can query the data from the index table.\n\n  * `:include_base_data`, specifies whether the index table includes the existing data in the base table, if set it to\n  `true` means the index includes the existing data, if set it to `false` means the index excludes the existing data,\n  optional, by default it is `true`.","ref":"ExAliyunOts.html#create_index/6-options","title":"Options - ExAliyunOts.create_index/6","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/295892.html) | [English](https://www.alibabacloud.com/help/doc-detail/295892.html)","ref":"ExAliyunOts.html#create_mapping_table/2","title":"ExAliyunOts.create_mapping_table/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    create_mapping_table(\"\"\"\n    CREATE TABLE table (\n      id VARCHAR(1024) PRIMARY KEY,\n      is_actived BOOL,\n      name MEDIUMTEXT,\n      score DOUBLE,\n      tags MEDIUMTEXT\n    )\n    \"\"\")","ref":"ExAliyunOts.html#create_mapping_table/2-example","title":"Example - ExAliyunOts.create_mapping_table/2","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/117452.html) | [English](https://www.alibabacloud.com/help/doc-detail/117452.html)","ref":"ExAliyunOts.html#create_search_index/4","title":"ExAliyunOts.create_search_index/4","type":"function"},{"doc":"import MyApp.TableStore\n\n    create_search_index \"table\", \"index_name\",\n      field_schemas: [\n        field_schema_keyword(\"name\"),\n        field_schema_integer(\"age\")\n      ]\n\n    create_search_index \"table\", \"index_name\",\n      field_schemas: [\n        field_schema_keyword(\"name\"),\n        field_schema_geo_point(\"location\"),\n        field_schema_integer(\"value\")\n      ]\n\n    create_search_index \"table\", \"index_name\",\n      field_schemas: [\n        field_schema_nested(\n        \"content\",\n        field_schemas: [\n          field_schema_keyword(\"header\"),\n          field_schema_keyword(\"body\")\n        ]\n      )\n    ]","ref":"ExAliyunOts.html#create_search_index/4-example","title":"Example - ExAliyunOts.create_search_index/4","type":"function"},{"doc":"* `:field_schemas`, required, a list of predefined search-index schema fields, please see the following helper functions:\n    - `ExAliyunOts.Search.field_schema_integer/2`\n    - `ExAliyunOts.Search.field_schema_float/2`\n    - `ExAliyunOts.Search.field_schema_boolean/2`\n    - `ExAliyunOts.Search.field_schema_keyword/2`\n    - `ExAliyunOts.Search.field_schema_text/2`\n    - `ExAliyunOts.Search.field_schema_nested/2`\n    - `ExAliyunOts.Search.field_schema_geo_point/2`\n  * `:index_sorts`, optional, a list of predefined sort-index schema fields, please see the following helper functions:\n    - `ExAliyunOts.Search.pk_sort/1`\n    - `ExAliyunOts.Search.field_sort/2`\n    - `ExAliyunOts.Search.geo_distance_sort/3`","ref":"ExAliyunOts.html#create_search_index/4-options","title":"Options - ExAliyunOts.create_search_index/4","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/27312.html) | [English](https://www.alibabacloud.com/help/doc-detail/27312.html)","ref":"ExAliyunOts.html#create_table/4","title":"ExAliyunOts.create_table/4","type":"function"},{"doc":"create_table \"table_name2\",\n      [{\"key1\", :string}, {\"key2\", :auto_increment}]\n\n    create_table \"table_name3\",\n      [{\"key1\", :string}],\n      reserved_throughput_write: 1,\n      reserved_throughput_read: 1,\n      time_to_live: 100_000,\n      max_versions: 3,\n      deviation_cell_version_in_sec: 6_400,\n      stream_spec: [is_enabled: true, expiration_time: 2]\n\n    create_table \"table_name\",\n      [{\"key1\", :string}],\n      defined_columns: [\n        {\"attr1\", :string},\n        {\"attr2\", :integer},\n        {\"attr3\", :boolean},\n        {\"attr4\", :double},\n        {\"attr5\", :binary}\n      ]\n\n    create_table \"table_name\",\n      [{\"key1\", :string}, {\"key2\", :string}],\n      defined_columns: [\n        {\"attr1\", :string},\n        {\"attr2\", :integer},\n        {\"attr3\", :boolean},\n        {\"attr4\", :double}\n      ]\n      index_metas: [\n        {\"index_global1\", [\"attr2\", \"key2\"], [\"attr1\", \"attr3\"]},\n        {\"index_global2\", [\"attr3\", \"key1\"], [\"attr1\", \"attr2\"], index_type: :global},\n        {\"index_local\", [\"key1\", \"attr1\"], [\"attr4\"], index_type: :local}\n      ]","ref":"ExAliyunOts.html#create_table/4-example","title":"Example - ExAliyunOts.create_table/4","type":"function"},{"doc":"* `:reserved_throughput_write`, optional, the reserved throughput write of table, by default it is 0.\n  * `:reserved_throughput_read`, optional, the reserved throughput read of table, by default it is 0.\n  * `time_to_live`, optional, the data storage time to live in seconds, the minimum settable value is 864_000 seconds (one day), by default it is -1 (for permanent).\n  * `:max_versions`, optional, the version of table, by default it is 1 that specifies there is only one version for columns.\n  * `:deviation_cell_version_in_sec`, optional, maximum version deviation, by default it is 864_000 seconds (one day).\n  * `:stream_spec`, specifies whether enable stream, by default it is not enable stream feature.\n    - `:is_enabled`, enable or not enable stream, use `true` or `false`;\n    - `:expiration_time`, the expiration time of stream.\n  * `:index_metas`, optional, the index meta of table, each item of `:index_metas` is in {String.t(), list(), list()} | {String.t(), list(), list(), Keyword.t()} format, by default it is [].\n  * `:defined_columns`, optional, the indexed attribute column, which is a combination of predefined columns of the base table, each item of `:defined_columns`\n  is in {String.t(), :integer | :double | :boolean | :string | :binary} format, by default it is [].\n  * `enable_local_txn`, optional, specifies whether to enable the local transaction feature. The value of this parameter is of the `:boolean` type. Default value: false. If you want to enable the local transaction feature when you create a data table, set this parameter to true.","ref":"ExAliyunOts.html#create_table/4-options","title":"Options - ExAliyunOts.create_table/4","type":"function"},{"doc":"","ref":"ExAliyunOts.html#delete_defined_columns/3","title":"ExAliyunOts.delete_defined_columns/3","type":"function"},{"doc":"import MyApp.TableStore\n\n    delete_defined_columns(\"table\", [\"attr1\", \"attr2\"])","ref":"ExAliyunOts.html#delete_defined_columns/3-example","title":"Example - ExAliyunOts.delete_defined_columns/3","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/94558.html) | [English](https://www.alibabacloud.com/help/doc-detail/94558.html)","ref":"ExAliyunOts.html#delete_index/3","title":"ExAliyunOts.delete_index/3","type":"function"},{"doc":"import MyApp.TableStore\n\n    delete_index(\"table_name\", \"index_name\")","ref":"ExAliyunOts.html#delete_index/3-example","title":"Example - ExAliyunOts.delete_index/3","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/27308.html) | [English](https://www.alibabacloud.com/help/doc-detail/27308.html)","ref":"ExAliyunOts.html#delete_row/4","title":"ExAliyunOts.delete_row/4","type":"function"},{"doc":"import MyApp.TableStore\n\n    delete_row \"table1\",\n      [{\"key1\", 3}, {\"key2\", \"3\"}],\n      condition: condition(:expect_exist, \"attr2\" == \"value2\")\n\n    delete_row \"table1\",\n      [{\"key1\", 3}, {\"key2\", \"3\"}],\n      condition: condition(:expect_exist, \"attr2\" == \"value2\"),\n      transaction_id: \"transaction_id\"","ref":"ExAliyunOts.html#delete_row/4-example","title":"Example - ExAliyunOts.delete_row/4","type":"function"},{"doc":"* `:condition`, required, please see `condition/1` or `condition/2` for details.\n  * `:transaction_id`, optional, write operation within local transaction.","ref":"ExAliyunOts.html#delete_row/4-options","title":"Options - ExAliyunOts.delete_row/4","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/117478.html) | [English](https://www.alibabacloud.com/help/doc-detail/117478.html)","ref":"ExAliyunOts.html#delete_search_index/3","title":"ExAliyunOts.delete_search_index/3","type":"function"},{"doc":"import MyApp.TableStore\n\n    delete_search_index(\"table\", \"index_name\")","ref":"ExAliyunOts.html#delete_search_index/3-example","title":"Example - ExAliyunOts.delete_search_index/3","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/27314.html) | [English](https://www.alibabacloud.com/help/doc-detail/27314.html)","ref":"ExAliyunOts.html#delete_table/2","title":"ExAliyunOts.delete_table/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    delete_table(\"table_name\")","ref":"ExAliyunOts.html#delete_table/2-example","title":"Example - ExAliyunOts.delete_table/2","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/295896.html) | [English](https://www.alibabacloud.com/help/doc-detail/295896.html)","ref":"ExAliyunOts.html#describe_mapping_table/2","title":"ExAliyunOts.describe_mapping_table/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    describe_mapping_table(\"table\")","ref":"ExAliyunOts.html#describe_mapping_table/2-example","title":"Example - ExAliyunOts.describe_mapping_table/2","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/117475.html) | [English](https://www.alibabacloud.com/help/doc-detail/117475.html)","ref":"ExAliyunOts.html#describe_search_index/3","title":"ExAliyunOts.describe_search_index/3","type":"function"},{"doc":"import MyApp.TableStore\n\n    describe_search_index(\"table\", \"index_name\")","ref":"ExAliyunOts.html#describe_search_index/3-example","title":"Example - ExAliyunOts.describe_search_index/3","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/27307.html) | [English](https://www.alibabacloud.com/help/doc-detail/27307.html)","ref":"ExAliyunOts.html#describe_table/2","title":"ExAliyunOts.describe_table/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    describe_table(table_name)","ref":"ExAliyunOts.html#describe_table/2-example","title":"Example - ExAliyunOts.describe_table/2","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/295893.html) | [English](https://www.alibabacloud.com/help/doc-detail/295893.html)","ref":"ExAliyunOts.html#drop_mapping_table/2","title":"ExAliyunOts.drop_mapping_table/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    drop_mapping_table(\"table\")","ref":"ExAliyunOts.html#drop_mapping_table/2-example","title":"Example - ExAliyunOts.drop_mapping_table/2","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/35193.html) | [English](https://www.alibabacloud.com/help/doc-detail/35193.html)","ref":"ExAliyunOts.html#filter/1","title":"ExAliyunOts.filter/1","type":"macro"},{"doc":"import MyApp.TableStore\n\n    get_row table_name1, [{\"key\", \"key1\"}],\n      columns_to_get: [\"name\", \"level\"],\n      filter: filter(\n        ({\"name\", ignore_if_missing: true, latest_version_only: true} == var_name and \"age\" > 1) or\n          (\"class\" == \"1\")\n      )\n\n    batch_get [\n      get(\n        table_name2,\n        [{\"key\", \"key1\"}],\n        filter: filter \"age\" >= 10\n      )\n    ]\n\n    put_row(table_name1, [{\"key\", \"key1\"}], [{\"type\", \"t:5\"}])\n\n    # Use `~r/\\d+/` regex expression to fetch the matched part (in this case it is \"5\") from\n    # the attribute column field, and then cast it into an integer for the \"==\" comparator.\n    #\n    get_row table_name1, [{\"key\", \"key1\"}],\n      filter: filter(\n        {\"type\", value_trans_rule: {~r/\\d+/, :integer}} == 5\n      )","ref":"ExAliyunOts.html#filter/1-example","title":"Example - ExAliyunOts.filter/1","type":"macro"},{"doc":"* `ignore_if_missing`, used when attribute column not existed.\n    * if a attribute column is not existed, when set `ignore_if_missing: true` in filter expression, there will ignore this row data in the returned result;\n    * if a attribute column is existed, the returned result won't be affected no matter true or false was set.\n  * `latest_version_only`, used when attribute column has multiple versions.\n    * if set `latest_version_only: true`, there will only check the value of the latest version is matched or not, by default it's set as `latest_version_only: true`;\n    * if set `latest_version_only: false`, there will check the value of all versions are matched or not.\n  * `value_trans_rule`, optional, a two-element tuple contains a `Regex` expression and one of [:integer, :double, :string] atom as a cast type, the regex expression\n    matched part will be cast into the corresponding type and then use it into the current condition comparator.","ref":"ExAliyunOts.html#filter/1-options","title":"Options - ExAliyunOts.filter/1","type":"macro"},{"doc":"Used in batch get operation, please see `batch_get/2` for details.","ref":"ExAliyunOts.html#get/3","title":"ExAliyunOts.get/3","type":"function"},{"doc":"The available options are same as `get_row/4`.","ref":"ExAliyunOts.html#get/3-options","title":"Options - ExAliyunOts.get/3","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/27309.html) | [English](https://www.alibabacloud.com/help/doc-detail/27309.html)","ref":"ExAliyunOts.html#get_range/5","title":"ExAliyunOts.get_range/5","type":"function"},{"doc":"import MyApp.TableStore\n\n    get_range \"table_name\",\n      [{\"key1\", 1}, {\"key2\", :inf_min}],\n      [{\"key1\", 4}, {\"key2\", :inf_max}],\n      direction: :forward\n\n    get_range \"table_name\",\n      [{\"key1\", 1}, {\"key2\", :inf_min}],\n      [{\"key1\", 4}, {\"key2\", :inf_max}],\n      time_range: {1525922253224, 1525923253224},\n      direction: :forward\n\n    get_range \"table_name\",\n      [{\"key1\", 1}, {\"key2\", :inf_min}],\n      [{\"key1\", 4}, {\"key2\", :inf_max}],\n      time_range: 1525942123224,\n      direction: :forward\n\nAlso, there is an alternative `stream_range/5` to iteratively get range of rows in stream.","ref":"ExAliyunOts.html#get_range/5-example","title":"Example - ExAliyunOts.get_range/5","type":"function"},{"doc":"* `:direction`, required, the order of fetch data, available options are `:forward` | `:backward`, by it is `:forward`.\n    - `:forward`, this query is performed in the order of primary key in ascending, in this case, input `inclusive_start_primary_keys` should less\n    than `exclusive_end_primary_keys`;\n    - `:backward`, this query is performed in the order of primary key in descending, in this case, input `inclusive_start_primary_keys` should greater\n    than `exclusive_end_primary_keys`.\n  * `:columns_to_get`, optional, fetch the special fields, by default it returns all fields, pass a field list to specify the expected return fields,\n    e.g. `[\"field1\", \"field2\"]`.\n  * `:start_column`, optional, specifies the start column when using for wide-row-read, the returned result contains this `:start_column`.\n  * `:end_column`, optional, specifies the end column when using for wide-row-read, the returned result does not contain this `:end_column`.\n  * `:filter`, optional, filter the return results in the server side, please see `filter/1` for details.\n  * `:max_versions`, optional, how many versions need to return in results, by default it is 1.\n  * `:transaction_id`, optional, read operation within local transaction.\n  * `:limit`, optional, the maximum number of rows of data to be returned, this value must be greater than 0, whether this option is set or not, there\n    returns a maximum of 5,000 data rows and the total data size never exceeds 4 MB.\n  * `:time_range`, optional, read data by timestamp range, support two ways to use it:\n    - `time_range: {start_timestamp, end_timestamp}`, the timestamp in the range (include `start_timestamp` but exclude `end_timestamp`)\n      and then will return in the results.\n    - `time_range: special_timestamp`, exactly match and then will return in the results.\n    - `:time_range` and `:max_versions` are mutually exclusive, by default use `max_versions: 1` and `time_range: nil`.","ref":"ExAliyunOts.html#get_range/5-options","title":"Options - ExAliyunOts.get_range/5","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/27305.html) | [English](https://www.alibabacloud.com/help/doc-detail/27305.html)","ref":"ExAliyunOts.html#get_row/4","title":"ExAliyunOts.get_row/4","type":"function"},{"doc":"import MyApp.TableStore\n\n    get_row \"table1\",\n      [{\"key1\", \"id1\"}, {\"key2\", \"id2\"}],\n      columns_to_get: [\"name\", \"level\"],\n      filter: filter((\"name[ignore_if_missing: true, latest_version_only: true]\" == var_name and \"age\" > 1) or (\"class\" == \"1\"))\n\n    get_row \"table2\",\n      [{\"key\", \"1\"}],\n      start_column: \"room\",\n      filter: pagination(offset: 0, limit: 3)\n\n    get_row \"table3\",\n      [{\"key\", \"1\"}],\n      transaction_id: \"transaction_id\"","ref":"ExAliyunOts.html#get_row/4-example","title":"Example - ExAliyunOts.get_row/4","type":"function"},{"doc":"* `:columns_to_get`, optional, fetch the special fields, by default it returns all fields, pass a field list to specify the expected return fields\n  e.g. `[\"field1\", \"field2\"]`.\n  * `:start_column`, optional, specifies the start column when using for wide-row-read, the returned result contains this `:start_column`.\n  * `:end_column`, optional, specifies the end column when using for wide-row-read, the returned result does not contain this `:end_column`.\n  * `:filter`, optional, filter the return results in the server side, please see `filter/1` for details.\n  * `:max_versions`, optional, how many versions need to return in results, by default it is 1.\n  * `:time_range`, optional, read data by timestamp range, support two ways to use it:\n    - `time_range: {start_timestamp, end_timestamp}`, the timestamp in the range (include `start_timestamp` but exclude `end_timestamp`)\n    and then will return in the results.\n    - `time_range: special_timestamp`, exactly match and then will return in the results.\n    - `:time_range` and `:max_versions` are mutually exclusive, by default use `max_versions: 1` and `time_range: nil`.\n  * `:transaction_id`, optional, read operation within local transaction.","ref":"ExAliyunOts.html#get_row/4-options","title":"Options - ExAliyunOts.get_row/4","type":"function"},{"doc":"As a wrapper built on `get_range/5` to fetch a full matched data set by iterate, if process a large items,\nrecommend to use `stream_range/5`.","ref":"ExAliyunOts.html#iterate_all_range/5","title":"ExAliyunOts.iterate_all_range/5","type":"function"},{"doc":"import MyApp.TableStore\n\n    iterate_all_range table_name1,\n      [{\"key1\", 1}, {\"key2\", :inf_min}],\n      [{\"key1\", 4}, {\"key2\", :inf_max}],\n      direction: :forward","ref":"ExAliyunOts.html#iterate_all_range/5-example","title":"Example - ExAliyunOts.iterate_all_range/5","type":"function"},{"doc":"Please see options of `get_range/5` for details.","ref":"ExAliyunOts.html#iterate_all_range/5-options","title":"Options - ExAliyunOts.iterate_all_range/5","type":"function"},{"doc":"A simple wrapper of `stream_parallel_scan/4` to take care `OTSSessionExpired` error with retry, make parallel scan\nas a stream that applies the given function to the complete result of scan query.\n\nIn general, recommend to use this function for the common use case of parallel scan.","ref":"ExAliyunOts.html#iterate_parallel_scan/5","title":"ExAliyunOts.iterate_parallel_scan/5","type":"function"},{"doc":"* `:scan_query`, required, the main option to use query.\n    - `:query`, required, bind to the query functions, the same as query option of `search/5`.\n    - `:limit`, optional, the limited size of query, defaults to 2000, the maximum value of limit is 2000.\n  * `:columns_to_get`, optional, fetch the special fields, by default it returns all fields of the search index, here are available options:\n    - `:all_from_index`, return all attribute column fields of search index;\n    - `:none`, do not return any attribute column fields;\n    - `[\"field1\", \"field2\"]`, specifies the expected return attribute column fields.\n  * `:timeout`, optional, the `:timeout` option of `Task.async_stream/3`, defaults to `:infinity`.","ref":"ExAliyunOts.html#iterate_parallel_scan/5-options","title":"Options - ExAliyunOts.iterate_parallel_scan/5","type":"function"},{"doc":"def iterate_stream(stream) do\n      Enum.map(stream, fn\n        {:ok, response} ->\n          response\n        {:error, error} ->\n          error\n      end)\n    end\n\n    iterate_parallel_scan(\n      \"table\",\n      \"index\",\n      &iterate_stream/1,\n      scan_query: [\n        query: match_query(\"is_actived\", \"true\"),\n        limit: 1000\n      ],\n      columns_to_get: [\"is_actived\", \"name\", \"score\"]\n    )","ref":"ExAliyunOts.html#iterate_parallel_scan/5-example","title":"Example - ExAliyunOts.iterate_parallel_scan/5","type":"function"},{"doc":"A simple wrapper of `stream_parallel_scan/4` to take care `OTSSessionExpired` error with retry, make parallel scan\nas a stream that applies the given function from `module` with the list of arguments `args` to the complete result of scan query.\n\nIn general, recommend to use this function for the common use case of parallel scan.","ref":"ExAliyunOts.html#iterate_parallel_scan/7","title":"ExAliyunOts.iterate_parallel_scan/7","type":"function"},{"doc":"Please see options of `iterate_parallel_scan/5`.","ref":"ExAliyunOts.html#iterate_parallel_scan/7-options","title":"Options - ExAliyunOts.iterate_parallel_scan/7","type":"function"},{"doc":"defmodule StreamHandler do\n      def iterate_stream(stream) do\n        Enum.map(stream, fn\n          {:ok, response} ->\n            response\n          {:error, error} ->\n            error\n        end)\n      end\n    end\n\n    iterate_parallel_scan(\n      \"table\",\n      \"index\",\n      StreamHandler,\n      :iterate_stream,\n      [],\n      scan_query: [\n        query: match_query(\"is_actived\", \"true\"),\n        limit: 1000\n      ],\n      columns_to_get: [\"field1\", \"field2\"]\n    )","ref":"ExAliyunOts.html#iterate_parallel_scan/7-example","title":"Example - ExAliyunOts.iterate_parallel_scan/7","type":"function"},{"doc":"As a wrapper built on `stream_search/4` to fetch a full matched data set as a stream, then use `Enum.reduce/2` to iteratively\nformat all data into a list, if process a large items, recommend to use `stream_search/4`.","ref":"ExAliyunOts.html#iterate_search/4","title":"ExAliyunOts.iterate_search/4","type":"function"},{"doc":"Please see options of `search/4` for details.","ref":"ExAliyunOts.html#iterate_search/4-options","title":"Options - ExAliyunOts.iterate_search/4","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/117477.html) | [English](https://www.alibabacloud.com/help/doc-detail/117477.html)","ref":"ExAliyunOts.html#list_search_index/2","title":"ExAliyunOts.list_search_index/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    list_search_index(\"table\")","ref":"ExAliyunOts.html#list_search_index/2-example","title":"Example - ExAliyunOts.list_search_index/2","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/27313.html) | [English](https://www.alibabacloud.com/help/doc-detail/27313.html)","ref":"ExAliyunOts.html#list_table/1","title":"ExAliyunOts.list_table/1","type":"function"},{"doc":"import MyApp.TableStore\n\n    list_table()","ref":"ExAliyunOts.html#list_table/1-example","title":"Example - ExAliyunOts.list_table/1","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/44573.html) | [English](https://www.alibabacloud.com/help/doc-detail/44573.html)","ref":"ExAliyunOts.html#pagination/1","title":"ExAliyunOts.pagination/1","type":"macro"},{"doc":"import MyApp.TableStore\n\n    get_row table_name,\n      [{\"key\", \"1\"}],\n      start_column: \"room\",\n      filter: pagination(offset: 0, limit: 3)\n\nUse `pagination/1` for `:filter` options when get row.","ref":"ExAliyunOts.html#pagination/1-example","title":"Example - ExAliyunOts.pagination/1","type":"macro"},{"doc":"Leverage concurrent tasks to query matched raw data (still be with search function) more quickly, in this use case, this function is improved for speed up\nscan query, but no guarantee to the order of query results, and does not support the aggregation of scan query.\n\nIn general, recommend to use `iterate_parallel_scan/5` or `iterate_parallel_scan/7` for the common use case of parallel scan.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/153862.html) | [English](https://www.alibabacloud.com/help/doc-detail/153862.htm)","ref":"ExAliyunOts.html#parallel_scan/4","title":"ExAliyunOts.parallel_scan/4","type":"function"},{"doc":"* `:scan_query`, required, the main option to use query.\n    - `:query`, required, bind to the query functions, the same as query option of `search/4`.\n    - `:limit`, optional, the limited size of query, defaults to 2000, the maximum value of limit is 2000.\n    - `:token`, optional, when do not load all the matched rows in a single request, there will return a `next_token` value in that result,\n    and then we can pass it to `:token` in the next same scan query to continue load the rest rows.\n    - `:max_parallel`, required, the maximum number of concurrent, as the `splits_size` value from the response of `compute_splits/3`.\n    - `:current_parallel_id`, required, refer the official document, the available value is in [0, max_parallel).\n  * `:columns_to_get`, optional, fetch the special fields, by default it returns all fields of the search index, here are available options:\n    - `:all_from_index`, return all attribute column fields of search index;\n    - `:none`, do not return any attribute column fields;\n    - `[\"field1\", \"field2\"]`, specifies the expected return attribute column fields.\n  * `session_id`, as usual, this option is required from the response of `compute_splits/3`, if not set this option, the query result may contain\n  duplicate data, refer the official document, once occurs an `OTSSessionExpired` error, must initiate another parallel scan task to re-query data.","ref":"ExAliyunOts.html#parallel_scan/4-options","title":"Options - ExAliyunOts.parallel_scan/4","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/27306.html) | [English](https://www.alibabacloud.com/help/doc-detail/27306.html)","ref":"ExAliyunOts.html#put_row/5","title":"ExAliyunOts.put_row/5","type":"function"},{"doc":"import MyApp.TableStore\n\n    put_row \"table1\",\n      [{\"key1\", \"id1\"}],\n      [{\"name\", \"name1\"}, {\"age\", 20}],\n      condition: :expect_not_exist,\n      return_type: :pk\n\n    put_row \"table2\",\n      [{\"key1\", \"id1\"}],\n      [{\"name\", \"name1\"}, {\"age\", 20}],\n      condition: :expect_not_exist,\n      transaction_id: \"transaction_id\",\n      return_type: :pk","ref":"ExAliyunOts.html#put_row/5-example","title":"Example - ExAliyunOts.put_row/5","type":"function"},{"doc":"* `:condition`, required, please see `condition/1` or `condition/2` for details.\n  * `:return_type`, optional, whether return the primary keys after put row, available options are `:pk` | `:none`, by default it is `:none`.\n  * `:transaction_id`, optional, write operation within local transaction.","ref":"ExAliyunOts.html#put_row/5-options","title":"Options - ExAliyunOts.put_row/5","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/295884.html) | [English](https://www.alibabacloud.com/help/doc-detail/295884.html)","ref":"ExAliyunOts.html#query/2","title":"ExAliyunOts.query/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    query(\"SELECT * FROM table LIMIT 20\")","ref":"ExAliyunOts.html#query/2-example","title":"Example - ExAliyunOts.query/2","type":"function"},{"doc":"The one entrance to use search index functions, please see `ExAliyunOts.Search` module for details.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/91974.html) | [English](https://www.alibabacloud.com/help/doc-detail/91974.html)","ref":"ExAliyunOts.html#search/4","title":"ExAliyunOts.search/4","type":"function"},{"doc":"* `:search_query`, required, the main option to use query and sort.\n    - `:query`, required, bind to the query functions:\n      - `ExAliyunOts.Search.bool_query/1`\n      - `ExAliyunOts.Search.exists_query/1`\n      - `ExAliyunOts.Search.geo_bounding_box_query/3`\n      - `ExAliyunOts.Search.geo_distance_query/3`\n      - `ExAliyunOts.Search.geo_polygon_query/2`\n      - `ExAliyunOts.Search.match_all_query/0`\n      - `ExAliyunOts.Search.match_phrase_query/2`\n      - `ExAliyunOts.Search.match_query/3`\n      - `ExAliyunOts.Search.nested_query/3`\n      - `ExAliyunOts.Search.prefix_query/2`\n      - `ExAliyunOts.Search.range_query/2`\n      - `ExAliyunOts.Search.term_query/2`\n      - `ExAliyunOts.Search.terms_query/2`\n      - `ExAliyunOts.Search.wildcard_query/2`\n    - `:sort`, optional, by default it is use `pk_sort/1`, bind to the Sort functions:\n      - `ExAliyunOts.Search.field_sort/2`\n      - `ExAliyunOts.Search.geo_distance_sort/3`\n      - `ExAliyunOts.Search.nested_filter/2`\n      - `ExAliyunOts.Search.pk_sort/1`\n      - `ExAliyunOts.Search.score_sort/1`\n    - `:aggs`, optional, please see official document in [Chinese](https://help.aliyun.com/document_detail/132191.html) | [English](https://www.alibabacloud.com/help/doc-detail/132191.html).\n    - `:group_bys`, optional, please see official document in [Chinese](https://help.aliyun.com/document_detail/132210.html) | [English](https://www.alibabacloud.com/help/doc-detail/132210.html).\n    - `:limit`, optional, the limited size of query.\n    - `:offset`, optional, the offset size of query. When the total rows are less or equal than 2000, can both used`:limit` and `:offset` to pagination.\n    - `:get_total_count`, optional, return the total count of the all matched rows, by default it is `true`.\n    - `:token`, optional, when do not load all the matched rows in a single request, there will return a `next_token` value in that result,\n    and then we can pass it to `:token` in the next same search query to continue load the rest rows.\n    - `:collapse`, optional, duplicate removal by the specified field, please see official document in [Chinese](https://help.aliyun.com/document_detail/154172.html), please NOTICE that currently there does not support use `:collapse` with `:token` together.\n  * `:columns_to_get`, optional, fetch the special fields, by default it returns all fields, here are available options:\n    - `:all`, return all attribute column fields;\n    - `:none`, do not return any attribute column fields;\n    - `[\"field1\", \"field2\"]`, specifies the expected return attribute column fields.","ref":"ExAliyunOts.html#search/4-options","title":"Options - ExAliyunOts.search/4","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/295884.html) | [English](https://www.alibabacloud.com/help/doc-detail/295884.html)","ref":"ExAliyunOts.html#sql_query/2","title":"ExAliyunOts.sql_query/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    sql_query(\"SELECT * FROM table LIMIT 20\")","ref":"ExAliyunOts.html#sql_query/2-example","title":"Example - ExAliyunOts.sql_query/2","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/93819.html) | [English](https://www.alibabacloud.com/help/doc-detail/93819.html)","ref":"ExAliyunOts.html#start_local_transaction/3","title":"ExAliyunOts.start_local_transaction/3","type":"function"},{"doc":"import MyApp.TableStore\n\n    partition_key = {\"key\", \"key1\"}\n    start_local_transaction(\"table\", partition_key)","ref":"ExAliyunOts.html#start_local_transaction/3-example","title":"Example - ExAliyunOts.start_local_transaction/3","type":"function"},{"doc":"Integrate `parallel_scan/4` with `compute_splits/3` as a complete use, base on the response of `compute_splits/3` to create the corresponding\nnumber of concurrency task(s), use `Task.async_stream/3` to make parallel scan as a stream which properly process `token`\nin every request of the internal, when use this function need to consider the possibility of the `OTSSessionExpired` error in the external.","ref":"ExAliyunOts.html#stream_parallel_scan/4","title":"ExAliyunOts.stream_parallel_scan/4","type":"function"},{"doc":"Please see options of `iterate_parallel_scan/5`.","ref":"ExAliyunOts.html#stream_parallel_scan/4-options","title":"Options - ExAliyunOts.stream_parallel_scan/4","type":"function"},{"doc":"As a wrapper built on `get_range/5` to create composable and lazy enumerable stream for iteration.","ref":"ExAliyunOts.html#stream_range/5","title":"ExAliyunOts.stream_range/5","type":"function"},{"doc":"import MyApp.TableStore\n\n    stream =\n      stream_range table_name1,\n        [{\"key1\", 1}, {\"key2\", :inf_min}],\n        [{\"key1\", 4}, {\"key2\", :inf_max}],\n        direction: :forward\n\n    Enum.to_list(stream, fn\n      {:ok, %{rows: rows} = response} ->\n        # process rows\n      {:error, error} ->\n        # occur error\n    end)","ref":"ExAliyunOts.html#stream_range/5-example","title":"Example - ExAliyunOts.stream_range/5","type":"function"},{"doc":"Please see options of `get_range/5` for details.","ref":"ExAliyunOts.html#stream_range/5-options","title":"Options - ExAliyunOts.stream_range/5","type":"function"},{"doc":"As a wrapper built on `search/4` to create composable and lazy enumerable stream for iteration.","ref":"ExAliyunOts.html#stream_search/4","title":"ExAliyunOts.stream_search/4","type":"function"},{"doc":"Please see options of `search/4` for details.","ref":"ExAliyunOts.html#stream_search/4-options","title":"Options - ExAliyunOts.stream_search/4","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/27307.html) | [English](https://www.alibabacloud.com/help/doc-detail/27307.html)","ref":"ExAliyunOts.html#update_row/4","title":"ExAliyunOts.update_row/4","type":"function"},{"doc":"import MyApp.TableStore\n\n    value = \"1\"\n    update_row \"table1\",\n      [{\"key1\", 2}, {\"key2\", \"2\"}],\n      delete: [{\"attr2\", nil, 1524464460}],\n      delete_all: [\"attr1\"],\n      put: [{\"attr3\", \"put_attr3\"}],\n      return_type: :pk,\n      condition: condition(:expect_exist, \"attr2\" == value)\n\n    update_row \"table2\",\n      [{\"key1\", 1}],\n      put: [{\"attr1\", \"put_attr1\"}],\n      increment: [{\"count\", 1}],\n      return_type: :after_modify,\n      return_columns: [\"count\"],\n      condition: :ignore\n\n    update_row \"table3\",\n      [partition_key],\n      put: [{\"new_attr1\", \"a1\"}],\n      delete_all: [\"level\", \"size\"],\n      condition: :ignore,\n      transaction_id: \"transaction_id\"","ref":"ExAliyunOts.html#update_row/4-example","title":"Example - ExAliyunOts.update_row/4","type":"function"},{"doc":"* `:put`, optional, require to be valid value, e.g. `[{\"field1\", \"value\"}, {...}]`, insert a new column if this field is not existed, or overwrite this field if existed.\n  * `:delete`, optional, delete the special version of a column or columns, please pass the column's version (timestamp) in `:delete` option, e.g. [{\"field1\", nil, 1524464460}, ...].\n  * `:delete_all`, optional, delete all versions of a column or columns, e.g. [\"field1\", \"field2\", ...].\n  * `:increment`, optional, attribute column(s) base on atomic counters for increment or decrement, require the value of column is integer.\n    - for increment, `increment: [{\"count\", 1}]`;\n    - for decrement, `increment: [{\"count\", -1}]`.\n  * `:return_type`, optional, whether return the primary keys after update row, available options are `:pk` | `:none` | `:after_modify`, by default it is `:none`.\n    - if use atomic counters, must set `return_type: :after_modify`.\n  * `:condition`, required, please see `condition/1` or `condition/2` for details.\n  * `:transaction_id`, optional, write operation within local transaction.","ref":"ExAliyunOts.html#update_row/4-options","title":"Options - ExAliyunOts.update_row/4","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/27315.html) | [English](https://www.alibabacloud.com/help/doc-detail/27315.html)","ref":"ExAliyunOts.html#update_table/3","title":"ExAliyunOts.update_table/3","type":"function"},{"doc":"import MyApp.TableStore\n\n    update_table \"table_name\",\n      reserved_throughput_write: 10,\n      time_to_live: 200_000,\n      stream_spec: [is_enabled: false]","ref":"ExAliyunOts.html#update_table/3-example","title":"Example - ExAliyunOts.update_table/3","type":"function"},{"doc":"Please see options of `create_table/4`.","ref":"ExAliyunOts.html#update_table/3-options","title":"Options - ExAliyunOts.update_table/3","type":"function"},{"doc":"Used in batch write operation, please see `batch_write/2` for details.","ref":"ExAliyunOts.html#write_delete/2","title":"ExAliyunOts.write_delete/2","type":"function"},{"doc":"The available operation same as `delete_row/4`.","ref":"ExAliyunOts.html#write_delete/2-options","title":"Options - ExAliyunOts.write_delete/2","type":"function"},{"doc":"Used in batch write operation, please see `batch_write/2` for details.","ref":"ExAliyunOts.html#write_put/3","title":"ExAliyunOts.write_put/3","type":"function"},{"doc":"The available options are same as `put_row/5`.","ref":"ExAliyunOts.html#write_put/3-options","title":"Options - ExAliyunOts.write_put/3","type":"function"},{"doc":"Used in batch write operation, please see `batch_write/2` for details.","ref":"ExAliyunOts.html#write_update/2","title":"ExAliyunOts.write_update/2","type":"function"},{"doc":"The available options are same as `update_row/4`.","ref":"ExAliyunOts.html#write_update/2-options","title":"Options - ExAliyunOts.write_update/2","type":"function"},{"doc":"","ref":"ExAliyunOts.html#t:column_name/0","title":"ExAliyunOts.column_name/0","type":"type"},{"doc":"","ref":"ExAliyunOts.html#t:exclusive_end_primary_keys/0","title":"ExAliyunOts.exclusive_end_primary_keys/0","type":"type"},{"doc":"","ref":"ExAliyunOts.html#t:inclusive_start_primary_keys/0","title":"ExAliyunOts.inclusive_start_primary_keys/0","type":"type"},{"doc":"","ref":"ExAliyunOts.html#t:index_name/0","title":"ExAliyunOts.index_name/0","type":"type"},{"doc":"","ref":"ExAliyunOts.html#t:instance/0","title":"ExAliyunOts.instance/0","type":"type"},{"doc":"","ref":"ExAliyunOts.html#t:options/0","title":"ExAliyunOts.options/0","type":"type"},{"doc":"","ref":"ExAliyunOts.html#t:primary_keys/0","title":"ExAliyunOts.primary_keys/0","type":"type"},{"doc":"","ref":"ExAliyunOts.html#t:query/0","title":"ExAliyunOts.query/0","type":"type"},{"doc":"","ref":"ExAliyunOts.html#t:result/0","title":"ExAliyunOts.result/0","type":"type"},{"doc":"","ref":"ExAliyunOts.html#t:row_existence/0","title":"ExAliyunOts.row_existence/0","type":"type"},{"doc":"","ref":"ExAliyunOts.html#t:table_name/0","title":"ExAliyunOts.table_name/0","type":"type"},{"doc":"","ref":"ExAliyunOts.DSL.html","title":"ExAliyunOts.DSL","type":"module"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/35194.html) | [English](https://www.alibabacloud.com/help/doc-detail/35194.html)","ref":"ExAliyunOts.DSL.html#condition/1","title":"ExAliyunOts.DSL.condition/1","type":"macro"},{"doc":"import MyApp.TableStore\n\n    update_row \"table\", [{\"pk\", \"pk1\"}],\n      delete_all: [\"attr1\", \"attr2\"],\n      return_type: :pk,\n      condition: condition(:expect_exist)\n\nThe available `existence` options: `:expect_exist` | `:expect_not_exist` | `:ignore`, here are some use cases for your reference:\n\nUse `condition(:expect_exist)`, expect the primary keys to row is existed.\n  * for `put_row/5`, if the primary keys have auto increment column type, meanwhile the target primary keys row is existed,\n  only use `condition(:expect_exist)` can successfully overwrite the row.\n  * for `update_row/4`, if the primary keys have auto increment column type, meanwhile the target primary keys row is existed,\n  only use `condition(:expect_exist)` can successfully update the row.\n  * for `delete_row/4`, no matter what primary keys type are, use `condition(:expect_exist)` can successfully delete the row.\n\nUse `condition(:expect_not_exist)`, expect the primary_keys to row is not existed.\n  * for `put_row/5`, if the primary keys have auto increment type,\n    - while the target primary keys row is existed, only use `condition(:expect_exist)` can successfully put the row;\n    - while the target primary keys row is not existed, only use `condition(:ignore)` can successfully put the row.\n\nUse `condition(:ignore)`, ignore the row existence check\n  * for `put_row/5`, if the primary keys have auto increment column type, meanwhile the target primary keys row is not existed,\n  only use `condition(:ignore)` can successfully put the row.\n  * for `update_row/4`, if the primary keys have auto increment column type, meanwhile the target primary keys row is not existed,\n  only use `condition(:ignore)` can successfully update the row.\n  * for `delete_row/4`, no matter what primary keys type are, use `condition(:ignore)` can successfully delete the row if existed.\n\nThe `batch_write/3` operation is a collection of put_row / update_row / delete_row operations.","ref":"ExAliyunOts.DSL.html#condition/1-example","title":"Example - ExAliyunOts.DSL.condition/1","type":"macro"},{"doc":"Similar to `condition/1` and support use filter expression (please see `filter/1`) as well, please refer them for details.","ref":"ExAliyunOts.DSL.html#condition/2","title":"ExAliyunOts.DSL.condition/2","type":"macro"},{"doc":"import MyApp.TableStore\n\n    delete_row \"table\",\n      [{\"key\", \"key1\"}, {\"key2\", \"key2\"}],\n      condition: condition(:expect_exist, \"attr_column\" == \"value2\")","ref":"ExAliyunOts.DSL.html#condition/2-example","title":"Example - ExAliyunOts.DSL.condition/2","type":"macro"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/35193.html) | [English](https://www.alibabacloud.com/help/doc-detail/35193.html)","ref":"ExAliyunOts.DSL.html#filter/1","title":"ExAliyunOts.DSL.filter/1","type":"macro"},{"doc":"import MyApp.TableStore\n\n    get_row table_name1, [{\"key\", \"key1\"}],\n      columns_to_get: [\"name\", \"level\"],\n      filter: filter(\n        ({\"name\", ignore_if_missing: true, latest_version_only: true} == var_name and \"age\" > 1) or\n          (\"class\" == \"1\")\n      )\n\n    batch_get [\n      get(\n        table_name2,\n        [{\"key\", \"key1\"}],\n        filter: filter \"age\" >= 10\n      )\n    ]\n\n    put_row(table_name1, [{\"key\", \"key1\"}], [{\"type\", \"t:5\"}])\n\n    # Use `~r/\\d+/` regex expression to fetch the matched part (in this case it is \"5\") from\n    # the attribute column field, and then cast it into an integer for the \"==\" comparator.\n    #\n    get_row table_name1, [{\"key\", \"key1\"}],\n      filter: filter(\n        {\"type\", value_trans_rule: {~r/\\d+/, :integer}} == 5\n      )","ref":"ExAliyunOts.DSL.html#filter/1-example","title":"Example - ExAliyunOts.DSL.filter/1","type":"macro"},{"doc":"* `ignore_if_missing`, used when attribute column not existed.\n    * if a attribute column is not existed, when set `ignore_if_missing: true` in filter expression, there will ignore this row data in the returned result;\n    * if a attribute column is existed, the returned result won't be affected no matter true or false was set.\n  * `latest_version_only`, used when attribute column has multiple versions.\n    * if set `latest_version_only: true`, there will only check the value of the latest version is matched or not, by default it's set as `latest_version_only: true`;\n    * if set `latest_version_only: false`, there will check the value of all versions are matched or not.\n  * `value_trans_rule`, optional, a two-element tuple contains a `Regex` expression and one of [:integer, :double, :string] atom as a cast type, the regex expression\n    matched part will be cast into the corresponding type and then use it into the current condition comparator.","ref":"ExAliyunOts.DSL.html#filter/1-options","title":"Options - ExAliyunOts.DSL.filter/1","type":"macro"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/44573.html) | [English](https://www.alibabacloud.com/help/doc-detail/44573.html)","ref":"ExAliyunOts.DSL.html#pagination/1","title":"ExAliyunOts.DSL.pagination/1","type":"macro"},{"doc":"import MyApp.TableStore\n\n    get_row table_name,\n      [{\"key\", \"1\"}],\n      start_column: \"room\",\n      filter: pagination(offset: 0, limit: 3)\n\nUse `pagination/1` for `:filter` options when get row.","ref":"ExAliyunOts.DSL.html#pagination/1-example","title":"Example - ExAliyunOts.DSL.pagination/1","type":"macro"},{"doc":"","ref":"ExAliyunOts.DSL.html#t:row_existence/0","title":"ExAliyunOts.DSL.row_existence/0","type":"type"},{"doc":"Describe request error from server http response.","ref":"ExAliyunOts.Error.html","title":"ExAliyunOts.Error","type":"module"},{"doc":"","ref":"ExAliyunOts.Error.html#t:t/0","title":"ExAliyunOts.Error.t/0","type":"type"},{"doc":"Describe Alibaba TableStore instance primary information.","ref":"ExAliyunOts.Instance.html","title":"ExAliyunOts.Instance","type":"module"},{"doc":"","ref":"ExAliyunOts.Instance.html#t:t/0","title":"ExAliyunOts.Instance.t/0","type":"type"},{"doc":"Use the multiple efficient index schemas of search index to solve complex query problems.\n\nHere are links to the search section of Alibaba official document: [Chinese](https://help.aliyun.com/document_detail/91974.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/91974.htm){:target=\"_blank\"}\n\nTo use `ExAliyunOts`, a module that calls `use ExAliyunOts` has to be defined:\n\n    defmodule MyApp.Tablestore do\n      use ExAliyunOts, instance: :my_instance\n    end\n\nThis automatically defines some search functions in `MyApp.Tablestore` module, we can use them as helpers when invoke `MyApp.Tablestore.search/3`, here are some examples:\n\n    import MyApp.Tablestore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: match_query(\"age\", 28),\n        sort: [\n          field_sort(\"age\", order: :desc)\n        ]\n      ]\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: exists_query(\"column_a\"),\n        group_bys: [\n          group_by_field(\"group_name\", \"column_b\",\n            sub_group_bys: [\n              group_by_range(\"group_name_1\", \"column_d\", [{0, 10}, {10, 20}])\n            ],\n            sort: [\n              group_key_sort(:desc)\n            ]\n          ),\n          group_by_field(\"group_name2\", \"column_c\")\n        ],\n        aggs: [\n          agg_min(\"aggregation_name\", \"column_e\")\n        ]\n      ]\n\nPlease notice:\n\n  * The statistics(via `:aggs`) and GroupBy type aggregations(via `:group_bys`) can be used at the same time.\n  * The GroupBy type aggregations support the nested sub statistics(via `:sub_aggs`) and sub GroupBy type aggregations(via `:sub_group_bys`).\n  * To ensure the performance and reduce the complexity of aggregations, there is a limitation with a certain number\n  of levels for nesting.\n  * If you are only care about using `:aggs` or `:group_bys`, meanwhile do not need the returned rows, you can set `:limit` as 0 to ignore the matched rows\n  return, there will have a better query performance.","ref":"ExAliyunOts.Search.html","title":"ExAliyunOts.Search","type":"module"},{"doc":"Calculate the average value of the assigned field by aggregation in the nested `:aggs` option of `:search_query`\noption in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/132191.html#title-7iv-ckn-sgq){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/132191.html#title-7iv-ckn-sgq){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#agg_avg/3","title":"ExAliyunOts.Search.agg_avg/3","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: ...,\n        aggs: [\n          agg_avg(\"agg_name\", \"score\")\n        ]\n      ]\n\nThe `aggregation_name` can be any business description string, when get the calculated results, we need to use\nit to fetch them.","ref":"ExAliyunOts.Search.html#agg_avg/3-example","title":"Example - ExAliyunOts.Search.agg_avg/3","type":"function"},{"doc":"* `:missing`, when the field is not existed in a row of data, if `:missing` is not set, the row will be ignored\n  in statistics; if `:missing` is set, the row will use `:missing` value to participate in the statistics of average\n  value, by default it's `nil` (not-set).","ref":"ExAliyunOts.Search.html#agg_avg/3-options","title":"Options - ExAliyunOts.Search.agg_avg/3","type":"function"},{"doc":"Calculate the count of the assigned field by aggregation in the nested `:aggs` option of `:search_query`\noption in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/132191.html#title-ams-asa-4if){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/132191.html#title-ams-asa-4if){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#agg_count/2","title":"ExAliyunOts.Search.agg_count/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: ...,\n        aggs: [\n          agg_sum(\"agg_name\", \"score\")\n        ]\n      ]\n\nThe `aggregation_name` can be any business description string, when get the calculated results, we need to use\nit to fetch them.\n\nIf the field is not existed in a row of data, then this row does not participate in the statistics of count.","ref":"ExAliyunOts.Search.html#agg_count/2-example","title":"Example - ExAliyunOts.Search.agg_count/2","type":"function"},{"doc":"Calculate the distinct count of the assigned field by aggregation in the nested `:aggs` option of `:search_query`\noption in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/132191.html#title-8ih-i28-959){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/132191.html#title-8ih-i28-959){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#agg_distinct_count/3","title":"ExAliyunOts.Search.agg_distinct_count/3","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: ...,\n        aggs: [\n          agg_distinct_count(\"agg_name\", \"score\")\n        ]\n      ]\n\nThe `aggregation_name` can be any business description string, when get the calculated results, we need to use\nit to fetch them.","ref":"ExAliyunOts.Search.html#agg_distinct_count/3-example","title":"Example - ExAliyunOts.Search.agg_distinct_count/3","type":"function"},{"doc":"* `:missing`, when the field is not existed in a row of data, if `:missing` is not set, the row will be ignored\n  in statistics; if `:missing` is set, the row will use `:missing` value to participate in the statistics of distinct \n  count, by default it's `nil` (not-set).","ref":"ExAliyunOts.Search.html#agg_distinct_count/3-options","title":"Options - ExAliyunOts.Search.agg_distinct_count/3","type":"function"},{"doc":"Calculate the maximum value of the assigned field by aggregation in the nested `:aggs` option of `:search_query`\noption in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/132191.html#title-u78-037-qw8){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/132191.html#title-u78-037-qw8){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#agg_max/3","title":"ExAliyunOts.Search.agg_max/3","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: ...,\n        aggs: [\n          agg_max(\"agg_name\", \"score\")\n        ]\n      ]\n\nThe `aggregation_name` can be any business description string, when get the calculated results, we need to use\nit to fetch them.","ref":"ExAliyunOts.Search.html#agg_max/3-example","title":"Example - ExAliyunOts.Search.agg_max/3","type":"function"},{"doc":"* `:missing`, when the field is not existed in a row of data, if `:missing` is not set, the row will be ignored\n  in statistics; if `:missing` is set, the row will use `:missing` value to participate in the statistics of maximum\n  value, by default it's `nil` (not-set).","ref":"ExAliyunOts.Search.html#agg_max/3-options","title":"Options - ExAliyunOts.Search.agg_max/3","type":"function"},{"doc":"Calculate the minimum value of the assigned field by aggregation in the nested `:aggs` option of `:search_query`\noption in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/132191.html#title-415-wbs-zqp){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/132191.html#title-415-wbs-zqp){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#agg_min/3","title":"ExAliyunOts.Search.agg_min/3","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: ...,\n        aggs: [\n          agg_min(\"agg_name\", \"score\")\n        ]\n      ]\n\nThe `aggregation_name` can be any business description string, when get the calculated results, we need to use\nit to fetch them.","ref":"ExAliyunOts.Search.html#agg_min/3-example","title":"Example - ExAliyunOts.Search.agg_min/3","type":"function"},{"doc":"* `:missing`, when the field is not existed in a row of data, if `:missing` is not set, the row will be ignored\n  in statistics; if `:missing` is set, the row will use `:missing` value to participate in the statistics of minimum\n  value, by default it's `nil` (not-set).","ref":"ExAliyunOts.Search.html#agg_min/3-options","title":"Options - ExAliyunOts.Search.agg_min/3","type":"function"},{"doc":"A percentile value indicates the relative position of a value in a dataset.\nFor example, when you collect statistics for the response time of each request during the routine O&M of your system, you must analyze the response time distribution by using percentiles such as p25, p50, p90, and p99.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/132191.html#title-hah-ppt-o6c){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/132191.html#title-hah-ppt-o6c){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#agg_percentiles/4","title":"ExAliyunOts.Search.agg_percentiles/4","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: ...,\n        aggs: [\n          agg_percentiles(\"agg_name\", \"score\", [0, 30, 50, 100], missing: 0.0)\n        ]\n      ]","ref":"ExAliyunOts.Search.html#agg_percentiles/4-example","title":"Example - ExAliyunOts.Search.agg_percentiles/4","type":"function"},{"doc":"* `:missing`, when the field is not existed in a row of data, if `:missing` is not set, the row will be ignored\n  in statistics; if `:missing` is set, the row will use `:missing` value to participate in the statistics of distinct\n  count, by default it's `nil` (not-set).","ref":"ExAliyunOts.Search.html#agg_percentiles/4-options","title":"Options - ExAliyunOts.Search.agg_percentiles/4","type":"function"},{"doc":"Calculate the summation of the assigned field by aggregation in the nested `:aggs` option of `:search_query`\noption in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/132191.html#title-885-937-2fz){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/132191.html#title-885-937-2fz){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#agg_sum/3","title":"ExAliyunOts.Search.agg_sum/3","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: ...,\n        aggs: [\n          agg_sum(\"agg_name\", \"score\")\n        ]\n      ]\n\nThe `aggregation_name` can be any business description string, when get the calculated results, we need to use\nit to fetch them.","ref":"ExAliyunOts.Search.html#agg_sum/3-example","title":"Example - ExAliyunOts.Search.agg_sum/3","type":"function"},{"doc":"* `:missing`, when the field is not existed in a row of data, if `:missing` is not set, the row will be ignored\n  in statistics; if `:missing` is set, the row will use `:missing` value to participate in the statistics of summation\n  value, by default it's `nil` (not-set).","ref":"ExAliyunOts.Search.html#agg_sum/3-options","title":"Options - ExAliyunOts.Search.agg_sum/3","type":"function"},{"doc":"After you group query results, you can query the rows in each group. This method can be used in a similar manner as ANY_VALUE(field) in MySQL.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/132191.html#title-w4s-aeo-28q){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/132191.html#title-w4s-aeo-28q){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#agg_top_rows/2","title":"ExAliyunOts.Search.agg_top_rows/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n        search_query: [\n          query: match_all_query(),\n          limit: 0,\n          group_bys: [\n            group_by_field(group_name, \"type\",\n              sub_aggs: [\n                agg_top_rows(sub_agg_name,\n                  limit: 2,\n                  sort: [\n                    field_sort(\"price\", order: :asc)\n                  ]\n                )\n              ]\n            )\n          ]\n       ]","ref":"ExAliyunOts.Search.html#agg_top_rows/2-example","title":"Example - ExAliyunOts.Search.agg_top_rows/2","type":"function"},{"doc":"* `:limit`, the maximum number of rows that can be returned for each group. By default, only 1 row of data is returned.\n  * `:sort`, the sorting method that is used to sort data in groups.","ref":"ExAliyunOts.Search.html#agg_top_rows/2-options","title":"Options - ExAliyunOts.Search.agg_top_rows/2","type":"function"},{"doc":"Use BoolQuery as the nested `:query` option of `:search_query` option in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/117498.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117498.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#bool_query/1","title":"ExAliyunOts.Search.bool_query/1","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: bool_query(\n          must: range_query(\"age\", from: 20, to: 32),\n          must_not: term_query(\"age\", 28)\n        )\n      ]\n\nThe following options can be a single `Query` or a list of `Query` to combine the \"And | Or | At least\"\nsearch condition.","ref":"ExAliyunOts.Search.html#bool_query/1-example","title":"Example - ExAliyunOts.Search.bool_query/1","type":"function"},{"doc":"* `:must`, specifies the Queries that the query result must match, this option is equivalent to the `AND` operator.\n  * `:must_not`, specifies the Queries that the query result must not match, this option is equivalent to the `NOT` operator.\n  * `:should`, specifies the Queries that the query result may or may not match, this option is equivalent to the `OR` operator.\n  * `:minimum_should_match`, specifies the minimum number of `:should` that the query result must match.","ref":"ExAliyunOts.Search.html#bool_query/1-options","title":"Options - ExAliyunOts.Search.bool_query/1","type":"function"},{"doc":"Use ExistsQuery as the nested `:query` option of `:search_query` option in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/124204.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/124204.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#exists_query/1","title":"ExAliyunOts.Search.exists_query/1","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: exists_query(\"values\")\n      ]","ref":"ExAliyunOts.Search.html#exists_query/1-example","title":"Example - ExAliyunOts.Search.exists_query/1","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/117453.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117453.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#field_schema_boolean/2","title":"ExAliyunOts.Search.field_schema_boolean/2","type":"function"},{"doc":"field_schema_boolean(\"status\")","ref":"ExAliyunOts.Search.html#field_schema_boolean/2-example","title":"Example - ExAliyunOts.Search.field_schema_boolean/2","type":"function"},{"doc":"* `:index`, specifies whether to set as index, by default it is true;\n  * `:enable_sort_and_agg`, specifies whether to support sort and statistics, by default it is true;\n  * `:store`, specifies whether to store the origin value in search index for a better read performance, by default it is true;\n  * `:is_array`, specifies whether the stored data is a JSON encoded list as a string, e.g. `\"[false,true,false]\"`.\n  * `:is_virtual_field`, specifies whether the field is a virtual column, by default it is false. This parameter is required only when you use virtual columns.\n  * `:source_field_name`, specifies the name of the source field to which the virtual column is mapped in the data table. This parameter is required when `:is_virtual_field` is set to true.","ref":"ExAliyunOts.Search.html#field_schema_boolean/2-options","title":"Options - ExAliyunOts.Search.field_schema_boolean/2","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/117453.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117453.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#field_schema_float/2","title":"ExAliyunOts.Search.field_schema_float/2","type":"function"},{"doc":"field_schema_float(\"price\")","ref":"ExAliyunOts.Search.html#field_schema_float/2-example","title":"Example - ExAliyunOts.Search.field_schema_float/2","type":"function"},{"doc":"* `:index`, specifies whether to set as index, by default it is true;\n  * `:enable_sort_and_agg`, specifies whether to support sort and statistics, by default it is true;\n  * `:store`, specifies whether to store the origin value in search index for a better read performance, by default it is true;\n  * `:is_array`, specifies whether the stored data is a JSON encoded list as a string, e.g. `\"[1.0,2.0]\"`.\n  * `:is_virtual_field`, specifies whether the field is a virtual column, by default it is false. This parameter is required only when you use virtual columns.\n  * `:source_field_name`, specifies the name of the source field to which the virtual column is mapped in the data table. This parameter is required when `:is_virtual_field` is set to true.","ref":"ExAliyunOts.Search.html#field_schema_float/2-options","title":"Options - ExAliyunOts.Search.field_schema_float/2","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/117453.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117453.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#field_schema_geo_point/2","title":"ExAliyunOts.Search.field_schema_geo_point/2","type":"function"},{"doc":"field_schema_geo_point(\"location\")","ref":"ExAliyunOts.Search.html#field_schema_geo_point/2-example","title":"Example - ExAliyunOts.Search.field_schema_geo_point/2","type":"function"},{"doc":"* `:index`, specifies whether to set as index, by default it is true;\n  * `:enable_sort_and_agg`, specifies whether to support sort and statistics, by default it is true;\n  * `:store`, specifies whether to store the origin value in search index for a better read performance, by default it is true;\n  * `:is_array`, specifies whether the stored data is a JSON encoded list as a string, e.g. `\"[\"10.21,10\",\"10.31,9.98\"]\"`.\n  * `:is_virtual_field`, specifies whether the field is a virtual column, by default it is false. This parameter is required only when you use virtual columns.\n  * `:source_field_name`, specifies the name of the source field to which the virtual column is mapped in the data table. This parameter is required when `:is_virtual_field` is set to true.","ref":"ExAliyunOts.Search.html#field_schema_geo_point/2-options","title":"Options - ExAliyunOts.Search.field_schema_geo_point/2","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/117453.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117453.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#field_schema_integer/2","title":"ExAliyunOts.Search.field_schema_integer/2","type":"function"},{"doc":"field_schema_integer(\"age\")","ref":"ExAliyunOts.Search.html#field_schema_integer/2-example","title":"Example - ExAliyunOts.Search.field_schema_integer/2","type":"function"},{"doc":"* `:index`, specifies whether to set as index, by default it is true;\n  * `:enable_sort_and_agg`, specifies whether to support sort and statistics, by default it is true;\n  * `:store`, specifies whether to store the origin value in search index for a better read performance, by default it is true;\n  * `:is_array`, specifies whether the stored data is a JSON encoded list as a string, e.g. `\"[1,2]\"`.\n  * `:is_virtual_field`, specifies whether the field is a virtual column, by default it is false. This parameter is required only when you use virtual columns.\n  * `:source_field_name`, specifies the name of the source field to which the virtual column is mapped in the data table. This parameter is required when `:is_virtual_field` is set to true.","ref":"ExAliyunOts.Search.html#field_schema_integer/2-options","title":"Options - ExAliyunOts.Search.field_schema_integer/2","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/117453.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117453.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#field_schema_keyword/2","title":"ExAliyunOts.Search.field_schema_keyword/2","type":"function"},{"doc":"field_schema_keyword(\"status\")","ref":"ExAliyunOts.Search.html#field_schema_keyword/2-example","title":"Example - ExAliyunOts.Search.field_schema_keyword/2","type":"function"},{"doc":"* `:index`, specifies whether to set as index, by default it is true;\n  * `:enable_sort_and_agg`, specifies whether to support sort and statistics, by default it is true;\n  * `:store`, specifies whether to store the origin value in search index for a better read performance, by default it is true;\n  * `:is_array`, specifies whether the stored data is a JSON encoded list as a string, e.g. `\"[\"a\",\"b\"]\"`.\n  * `:is_virtual_field`, specifies whether the field is a virtual column, by default it is false. This parameter is required only when you use virtual columns.\n  * `:source_field_name`, specifies the name of the source field to which the virtual column is mapped in the data table. This parameter is required when `:is_virtual_field` is set to true.","ref":"ExAliyunOts.Search.html#field_schema_keyword/2-options","title":"Options - ExAliyunOts.Search.field_schema_keyword/2","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/117453.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117453.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#field_schema_nested/2","title":"ExAliyunOts.Search.field_schema_nested/2","type":"function"},{"doc":"field_schema_nested(\n      \"content\",\n      field_schemas: [\n        field_schema_keyword(\"header\"),\n        field_schema_keyword(\"body\"),\n      ]","ref":"ExAliyunOts.Search.html#field_schema_nested/2-example","title":"Example - ExAliyunOts.Search.field_schema_nested/2","type":"function"},{"doc":"* `:field_schemas`, required, the nested field schema(s);\n  * `:index`, specifies whether to set as index, by default it is true;\n  * `:store`, specifies whether to store the origin value in search index for a better read performance, by default it is true;\n  * `:enable_sort_and_agg`, specifies whether to support sort and statistics, by default it is true.","ref":"ExAliyunOts.Search.html#field_schema_nested/2-options","title":"Options - ExAliyunOts.Search.field_schema_nested/2","type":"function"},{"doc":"Official document in [Chinese](https://help.aliyun.com/document_detail/117453.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117453.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#field_schema_text/2","title":"ExAliyunOts.Search.field_schema_text/2","type":"function"},{"doc":"field_schema_text(\"content\")","ref":"ExAliyunOts.Search.html#field_schema_text/2-example","title":"Example - ExAliyunOts.Search.field_schema_text/2","type":"function"},{"doc":"* `:index`, specifies whether to set as index, by default it is true;\n  * `:store`, specifies whether to store the origin value in search index for a better read performance, by default it is true;\n  * `:is_array`, specifies whether the stored data is a JSON encoded list as a string, e.g. `\"[\"a\",\"b\"]\"`.\n  * `:analyzer`, optional, please see analyzer document in [Chinese](https://help.aliyun.com/document_detail/120227.html){:target=\"_blank\"} |\n  [English](https://www.alibabacloud.com/help/en/doc-detail/120227.html){:target=\"_blank\"}.\n  * `:analyzer_parameter`, optional, please see analyzer document in [Chinese](https://help.aliyun.com/document_detail/120227.html){:target=\"_blank\"} |\n  [English](https://www.alibabacloud.com/help/en/doc-detail/120227.html){:target=\"_blank\"}.\n  * `:is_virtual_field`, specifies whether the field is a virtual column, by default it is false. This parameter is required only when you use virtual columns.\n  * `:source_field_name`, specifies the name of the source field to which the virtual column is mapped in the data table. This parameter is required when `:is_virtual_field` is set to true.","ref":"ExAliyunOts.Search.html#field_schema_text/2-options","title":"Options - ExAliyunOts.Search.field_schema_text/2","type":"function"},{"doc":"Sort by the value of a column, use it in the nested `:sort` option of `:search_query` option in `ExAliyunOts.search/4`.","ref":"ExAliyunOts.Search.html#field_sort/2","title":"ExAliyunOts.Search.field_sort/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: ...,\n        sort: [\n          field_sort(\"field_a\", order: :desc)\n        ]\n      ]\n\nIf there's a nested type of search index, and they are a integer or float list, we can use `:mode` to\nsort according to the minimum/maximum/average value of the list, by default it's `:nil`.\n\nFor example, there's a nested type as \"values\" field, the following query will find \"values\" field existed\nas matched rows, and sort by the minimum value of list items.","ref":"ExAliyunOts.Search.html#field_sort/2-example","title":"Example - ExAliyunOts.Search.field_sort/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: exists_query(\"values\"),\n        sort: [\n          field_sort(\"values\", mode: :min)\n        ]\n      ]\n\nStill for nested type of search index, we can sort by the nested value via `:nested_filter` option, for example,\nsort by the value of \"content.header\" in `:desc` order.","ref":"ExAliyunOts.Search.html#field_sort/2-example","title":"Example - ExAliyunOts.Search.field_sort/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: nested_query(\n          \"content\",\n          [\n            exists_query(\"content.header\")\n          ]\n        ),\n        sort: [\n          field_sort(\"content.header\",\n            order: :desc,\n            nested_filter: nested_filter(\n              \"content\",\n              prefix_query(\"content.header\", \"header\")\n            )\n          )\n        ]\n      ]\n\nPlease ensure that the query criteria matched will participate in sorting, if there exists any not matched case\nwill lead to uncertainty of sorting results.","ref":"ExAliyunOts.Search.html#field_sort/2-example","title":"Example - ExAliyunOts.Search.field_sort/2","type":"function"},{"doc":"* `:mode`, optional, available options are `:min` | `:max` | `:avg`, by default it's `:nil`;\n  * `:order`, optional, available options are `:asc` | `:desc`, by default it's `:asc`;\n  * `:nested_filter`, optional, see `nested_filter/2` for details.","ref":"ExAliyunOts.Search.html#field_sort/2-options","title":"Options - ExAliyunOts.Search.field_sort/2","type":"function"},{"doc":"Use GeoBoundingBoxQuery as the nested `:query` option of `:search_query` option in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/117499.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117499.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#geo_bounding_box_query/3","title":"ExAliyunOts.Search.geo_bounding_box_query/3","type":"function"},{"doc":"import MyApp.TableStore\n    search \"table\", \"index_name\",\n      search_query: [\n        query: geo_bounding_box_query(\"location\", \"10,-10\", \"-10,10\")\n      ]\n\nPlease notice that all geographic coordinates are in \"$latitude,$longitude\" format.","ref":"ExAliyunOts.Search.html#geo_bounding_box_query/3-example","title":"Example - ExAliyunOts.Search.geo_bounding_box_query/3","type":"function"},{"doc":"Use GeoDistanceQuery as the nested `:query` option of `:search_query` option in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/117500.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117500.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#geo_distance_query/3","title":"ExAliyunOts.Search.geo_distance_query/3","type":"function"},{"doc":"import MyApp.TableStore\n    search \"table\", \"index_name\",\n      search_query: [\n        query: geo_distance_query(\"location\", 500_000, \"5,5\")\n      ]\n\nPlease notice that all geographic coordinates are in \"$latitude,$longitude\" format.","ref":"ExAliyunOts.Search.html#geo_distance_query/3-example","title":"Example - ExAliyunOts.Search.geo_distance_query/3","type":"function"},{"doc":"Geographic distance sorting, according to the sum of distances between to the input geographical points,\nsort by the minimum/maximum/average summation value.","ref":"ExAliyunOts.Search.html#geo_distance_sort/3","title":"ExAliyunOts.Search.geo_distance_sort/3","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: geo_distance_query(\"location\", 500_000, \"5,5\"),\n        sort: [\n          geo_distance_sort(\"location\", [\"5.14,5.21\"], order: :asc)\n        ]\n      ]\n\nThe input points are a list of string, each format as \"$latitude,$longitude\".","ref":"ExAliyunOts.Search.html#geo_distance_sort/3-example","title":"Example - ExAliyunOts.Search.geo_distance_sort/3","type":"function"},{"doc":"* `:order`, optional, available options are `:asc` | `:desc`;\n  * `:mode`, optional, used for nested type field within integer or float, as `:min` will sort by the minimum value of\n  items, as `:max` will sort by the maximum value of items, as `:avg` will sort by the average value of items, by default\n  it's `:nil`;\n  * `:distance_type`, optional, available options are `:arc` | `:plane`, as `:arc` means distance calculated by arc surface, as `:plane` means distance calculated by plane.","ref":"ExAliyunOts.Search.html#geo_distance_sort/3-options","title":"Options - ExAliyunOts.Search.geo_distance_sort/3","type":"function"},{"doc":"Use GeoPolygonQuery as the nested `:query` option of `:search_query` option in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/117501.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117501.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#geo_polygon_query/2","title":"ExAliyunOts.Search.geo_polygon_query/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: geo_polygon_query(\"location\", [\"11,11\", \"0,0\", \"1,5\"])\n      ]\n\nPlease notice that all geographic coordinates are in \"$latitude,$longitude\" format.","ref":"ExAliyunOts.Search.html#geo_polygon_query/2-example","title":"Example - ExAliyunOts.Search.geo_polygon_query/2","type":"function"},{"doc":"The `:group_bys` results are grouped according to the value of a field, the same value will be put into a group, finally, \nthe value of each group and the number corresponding to the value will be returned.\n\nWe can set it in the nested `:group_bys` option of `:search_query` option in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/132191.html#title-w51-5lt-5hd){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/132191.html#title-w51-5lt-5hd){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#group_by_field/3","title":"ExAliyunOts.Search.group_by_field/3","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: ...,\n        group_bys: [\n          group_by_field(\"group_name\", \"type\",\n            size: 3,\n            sub_group_bys: [\n              group_by_field(\"sub_gn1\", \"is_actived\")\n            ],\n            sort: [\n              row_count_sort(:asc),\n              group_key_sort(:desc)\n            ]\n          ),\n          group_by_field(\"group_name2\", \"is_actived\")\n        ]\n      ]\n\nThe `group_name` can be any business description string, when get the grouped results, we need to use\nit to fetch them.","ref":"ExAliyunOts.Search.html#group_by_field/3-example","title":"Example - ExAliyunOts.Search.group_by_field/3","type":"function"},{"doc":"* `:sort`, optional, add sorting rules for items in a group, by default, sort in descending order according to \n  the quantity of items in the group. Support `group_key_sort/1` | `row_count_sort/1` | `sub_agg_sort/2` sort.\n  * `:size`, optional, the number of returned groups.\n  * `:sub_group_bys`, optional, add sub GroupBy type aggregations.\n  * `:sub_aggs`, optional, add sub statistics.","ref":"ExAliyunOts.Search.html#group_by_field/3-options","title":"Options - ExAliyunOts.Search.group_by_field/3","type":"function"},{"doc":"On the query results, group by filters (they're `Query` usecase), and then get the number of matched filters,\nthe order of the returned results is the same as that of the added filter(s).\n\nWe can set it in the nested `:group_bys` option of `:search_query` option in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/132191.html#title-yj6-4mq-xjf){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/132191.html#title-yj6-4mq-xjf){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#group_by_filter/3","title":"ExAliyunOts.Search.group_by_filter/3","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: ...,\n        group_bys: [\n          group_by_filter(\n            \"group_name\",\n            [\n              term_query(\"is_actived\", true),\n              range_query(\"price\", from: 50)\n            ]\n          )\n        ]\n      ]","ref":"ExAliyunOts.Search.html#group_by_filter/3-example","title":"Example - ExAliyunOts.Search.group_by_filter/3","type":"function"},{"doc":"* `:sub_group_bys`, optional, add sub GroupBy type aggregations.\n  * `:sub_aggs`, optional, add sub statistics.","ref":"ExAliyunOts.Search.html#group_by_filter/3-options","title":"Options - ExAliyunOts.Search.group_by_filter/3","type":"function"},{"doc":"The query results are grouped according to the range from a certain center Geo point, if the distance difference\nis within a certain range, it will be put into a group, and finally the number of corresponding items in each\nrange will be returned.\n\nWe can set it in the nested `:group_bys` option of `:search_query` option in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/132191.html#title-gk2-iw8-4rj){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/132191.html#title-gk2-iw8-4rj){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#group_by_geo_distance/4","title":"ExAliyunOts.Search.group_by_geo_distance/4","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: ...,\n        group_bys: [\n          group_by_geo_distance(\"test\", \"location\",\n            [\n              {0, 100_000},\n              {100_000, 500_000},\n              {500_000, 1000_000},\n            ],\n            lon: 0,\n            lat: 0,\n            sub_aggs: [\n              agg_sum(\"test_sum\", \"value\")\n            ]\n          )\n        ]\n      ]","ref":"ExAliyunOts.Search.html#group_by_geo_distance/4-example","title":"Example - ExAliyunOts.Search.group_by_geo_distance/4","type":"function"},{"doc":"* `:lon`, required, the longitude of the origin center point, integer or float.\n  * `:lat`, required, the latitude of the origin center point, integer or float.\n  * `:sub_group_bys`, optional, add sub GroupBy type aggregations.\n  * `:sub_aggs`, optional, add sub statistics.","ref":"ExAliyunOts.Search.html#group_by_geo_distance/4-options","title":"Options - ExAliyunOts.Search.group_by_geo_distance/4","type":"function"},{"doc":"The aggregation method that can be used to group query results based on specific data intervals.\nField values that are within the same range are grouped together.\nThe value range of each group and the number of values in each group are returned.\n\nWe can set it in the nested `:group_bys` option of `:search_query` option in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/132191.html#title-kb5-4qw-xt6){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/132191.html#title-kb5-4qw-xt6){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#group_by_histogram/5","title":"ExAliyunOts.Search.group_by_histogram/5","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: ...,\n        group_bys: [\n          group_by_histogram(\n            \"group_name\",\n            \"price\",        # histogram by field `price(double)`\n            10.0,           # every `10.0` is a group\n            {0.0, 100.0},   # from `0.0` to `100.0`\n            missing: 0.0    # if `price` of a row is missing, the value will be treated as `0.0`\n          )\n        ]\n      ]","ref":"ExAliyunOts.Search.html#group_by_histogram/5-example","title":"Example - ExAliyunOts.Search.group_by_histogram/5","type":"function"},{"doc":"* `:missing`, optional, the default value for the field that is used to perform the aggregation operation on a row when the field value is empty.\n  * `:min_doc_count`, optional, the minimum number of rows. If the number of rows in a group is less than the minimum number of rows, the aggregation results for the group are not returned.","ref":"ExAliyunOts.Search.html#group_by_histogram/5-options","title":"Options - ExAliyunOts.Search.group_by_histogram/5","type":"function"},{"doc":"The `:group_bys` results are grouped according to the range of a field, if the field value is within a range,\nit will be put into a group, finally, the number corresponding to the value will be returned.\n\nWe can set it in the nested `:group_bys` option of `:search_query` option in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/132191.html#title-6dl-40u-1j8){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/132191.html#title-6dl-40u-1j8){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#group_by_range/4","title":"ExAliyunOts.Search.group_by_range/4","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: ...,\n        group_bys: [\n          group_by_range(\"group_name\", \"price\",\n            [\n              {0, 18},\n              {18, 50}\n            ],\n            sub_group_bys: [\n              group_by_field(\"sorted_by_type\", \"type\",\n                sort: [\n                  group_key_sort(:asc)\n                ]\n              )\n            ],\n            sub_aggs: [\n              agg_distinct_count(\"distinct_price\", \"price\")\n            ]\n          )\n        ]\n      ]\n\nThe `group_name` can be any business description string, when get the grouped results, we need to use\nit to fetch them.\n\nPlease notice that each range item(as a tuple, according to {`from`, `to`}) of `ranges`, its start is greater\nthan or equal to `from`, and its ending is less than `to`, the range interval value can be integer or float.","ref":"ExAliyunOts.Search.html#group_by_range/4-example","title":"Example - ExAliyunOts.Search.group_by_range/4","type":"function"},{"doc":"* `:sub_group_bys`, optional, add sub GroupBy type aggregations.\n  * `:sub_aggs`, optional, add sub statistics.","ref":"ExAliyunOts.Search.html#group_by_range/4-options","title":"Options - ExAliyunOts.Search.group_by_range/4","type":"function"},{"doc":"Use in `group_by_field/3` scenario, in ascending/descending order of field literal.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/132191.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/132191.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#group_key_sort/1","title":"ExAliyunOts.Search.group_key_sort/1","type":"function"},{"doc":"In the following example, the returned results will be sorted in descending order of the `\"type\"` field:\n\n    import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: ...,\n        group_bys: [\n          group_by_field(\n            \"group_name\",\n            \"type\",\n            sub_group_bys: [\n              ...\n            ],\n            sort: [\n              group_key_sort(:desc)\n            ]\n          )\n        ]\n      ]","ref":"ExAliyunOts.Search.html#group_key_sort/1-example","title":"Example - ExAliyunOts.Search.group_key_sort/1","type":"function"},{"doc":"","ref":"ExAliyunOts.Search.html#map_scan_options/2","title":"ExAliyunOts.Search.map_scan_options/2","type":"function"},{"doc":"Use MatchAllQuery as the nested `:query` option of `:search_query` option in `ExAliyunOts.search/4`.","ref":"ExAliyunOts.Search.html#match_all_query/0","title":"ExAliyunOts.Search.match_all_query/0","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: match_all_query()\n      ]\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/117484.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117484.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#match_all_query/0-example","title":"Example - ExAliyunOts.Search.match_all_query/0","type":"function"},{"doc":"Use MatchPhraseQuery as the nested `:query` option of `:search_query` option in `ExAliyunOts.search/4`.\n\nSimilar to `MatchQuery`, however, the location relationship of multiple terms after word segmentation will be considered,\nmultiple terms after word segmentation must exist in the same order and location in the row data to be hit this query.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/117486.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117486.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#match_phrase_query/2","title":"ExAliyunOts.Search.match_phrase_query/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: match_phrase_query(\"content\", \"tablestore\")\n      ]","ref":"ExAliyunOts.Search.html#match_phrase_query/2-example","title":"Example - ExAliyunOts.Search.match_phrase_query/2","type":"function"},{"doc":"Use MatchQuery as the nested `:query` option of `:search_query` option in `ExAliyunOts.search/4`.","ref":"ExAliyunOts.Search.html#match_query/3","title":"ExAliyunOts.Search.match_query/3","type":"function"},{"doc":"import MyApp.TableStore\n    search \"table\", \"index_name\",\n      search_query: [\n        query: match_query(\"age\", 28)\n      ]\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/117485.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117485.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#match_query/3-example","title":"Example - ExAliyunOts.Search.match_query/3","type":"function"},{"doc":"* `:minimum_should_match`, the minimum number of terms that the value of the fieldName field in a row\n  contains when Table Store returns this row in the query result, by default it's 1.\n  * `:operator`, the operator used in a logical operation, by default it's `Or`, it means that as long as several\n  terms after the participle are partially hit, they are considered hit this query.","ref":"ExAliyunOts.Search.html#match_query/3-options","title":"Options - ExAliyunOts.Search.match_query/3","type":"function"},{"doc":"Use for the nested type field in `field_sort/2` as `:nested_filter` option, the input `filter`\nis a Query to filter results.","ref":"ExAliyunOts.Search.html#nested_filter/2","title":"ExAliyunOts.Search.nested_filter/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: nested_query(\n          \"content\",\n          [\n            exists_query(\"content.header\")\n          ]\n        ),\n        sort: [\n          field_sort(\"content.header\",\n            order: :desc,\n            nested_filter: nested_filter(\n              \"content\",\n              prefix_query(\"content.header\", \"header\")\n            )\n          )\n        ]\n      ]\n\nPlease ensure that the query criteria matched will participate in sorting, if there exists any not matched case\nwill lead to uncertainty of sorting results.","ref":"ExAliyunOts.Search.html#nested_filter/2-example","title":"Example - ExAliyunOts.Search.nested_filter/2","type":"function"},{"doc":"Use NestedQuery as the nested `:query` option of `:search_query` option in `ExAliyunOts.search/4`, the target field\nneed to be a nested type, it is used to query sub documents of nested type.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/120221.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/120221.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#nested_query/3","title":"ExAliyunOts.Search.nested_query/3","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: nested_query(\n          \"content\",\n          term_query(\"content.header\", \"header1\")\n        )\n      ]","ref":"ExAliyunOts.Search.html#nested_query/3-example","title":"Example - ExAliyunOts.Search.nested_query/3","type":"function"},{"doc":"* `:score_mode`, available options have `:none` | `:avg` | `:max` | `:total` | `:min`, by default\n  it's `:none`.","ref":"ExAliyunOts.Search.html#nested_query/3-options","title":"Options - ExAliyunOts.Search.nested_query/3","type":"function"},{"doc":"Sort by the primary key(s) of row, use it in the nested `:sort` option of `:search_query` option in `ExAliyunOts.search/4`.\n\nEach search request use this sort by default.","ref":"ExAliyunOts.Search.html#pk_sort/1","title":"ExAliyunOts.Search.pk_sort/1","type":"function"},{"doc":"Use PrefixQuery as the nested `:query` option of `:search_query` option in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/117495.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117495.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#prefix_query/2","title":"ExAliyunOts.Search.prefix_query/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: prefix_query(\"name\", \"n\")\n      ]","ref":"ExAliyunOts.Search.html#prefix_query/2-example","title":"Example - ExAliyunOts.Search.prefix_query/2","type":"function"},{"doc":"Use RangeQuery as the nested `:query` option of `:search_query` option in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/117496.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117496.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#range_query/1","title":"ExAliyunOts.Search.range_query/1","type":"macro"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: range_query(1 <= \"score\" and \"score\" <= 10)\n      ]","ref":"ExAliyunOts.Search.html#range_query/1-example","title":"Example - ExAliyunOts.Search.range_query/1","type":"macro"},{"doc":"range_query(\"score\" > 1)\n    range_query(\"score\" >= 1)\n    range_query(\"score\" < 10)\n    range_query(\"score\" <= 10)\n    range_query(1 < \"score\" and \"score\" < 10)\n    range_query(1 <= \"score\" and \"score\" <= 10)","ref":"ExAliyunOts.Search.html#range_query/1-supports","title":"Supports - ExAliyunOts.Search.range_query/1","type":"macro"},{"doc":"Use RangeQuery as the nested `:query` option of `:search_query` option in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/117496.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117496.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#range_query/2","title":"ExAliyunOts.Search.range_query/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: range_query(\n          \"score\",\n          from: 60,\n          to: 80,\n          include_upper: false,\n          include_lower: false\n        )\n      ]\n\n  # or support Range\n\n  search \"table\", \"index_name\",\n    search_query: [\n      query: range_query(\"score\", 60..80)\n    ]\n\n  # equal to\n\n  search \"table\", \"index_name\",\n    search_query: [\n      query: range_query(\"score\", from: 60, to: 80)\n    ]","ref":"ExAliyunOts.Search.html#range_query/2-example","title":"Example - ExAliyunOts.Search.range_query/2","type":"function"},{"doc":"* `:from`, the value of the start position.\n  * `:to`, the value of the end position.\n  * `:include_lower`, specifies whether to include the `:from` value in the result, available options are `true` | `false`,\n  by default it's `true`.\n  * `:include_upper`, specifies whether to include the `:to` value in the result, available options are `true` | `false`,\n  by default it's `true`.","ref":"ExAliyunOts.Search.html#range_query/2-options","title":"Options - ExAliyunOts.Search.range_query/2","type":"function"},{"doc":"Use in `group_by_field/3` scenario, in ascending/descending order of row(s) count.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/132191.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/132191.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#row_count_sort/1","title":"ExAliyunOts.Search.row_count_sort/1","type":"function"},{"doc":"In the following example, the returned results will be sorted in ascending order of the matched row(s):\n\n    import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: ...,\n        group_bys: [\n          group_by_field(\n            \"group_name\",\n            \"type\",\n            sub_group_bys: [\n              ...\n            ],\n            sort: [\n              row_count_sort(:asc)\n            ]\n          )\n        ]\n      ]","ref":"ExAliyunOts.Search.html#row_count_sort/1-example","title":"Example - ExAliyunOts.Search.row_count_sort/1","type":"function"},{"doc":"Sort by the relevance score to apply the full-text indexing properly, use it in the nested `:sort` option of\n`:search_query` option in `ExAliyunOts/search/4`.","ref":"ExAliyunOts.Search.html#score_sort/1","title":"ExAliyunOts.Search.score_sort/1","type":"function"},{"doc":"Use in `group_by_field/3` scenario, in ascending/descending order of the value from sub statistics.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/132191.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/132191.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#sub_agg_sort/2","title":"ExAliyunOts.Search.sub_agg_sort/2","type":"function"},{"doc":"Use TermQuery as the nested `:query` option of `:search_query` option in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/117488.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117488.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#term_query/2","title":"ExAliyunOts.Search.term_query/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: term_query(\"age\", 28)\n      ]","ref":"ExAliyunOts.Search.html#term_query/2-example","title":"Example - ExAliyunOts.Search.term_query/2","type":"function"},{"doc":"Use TermsQuery as the nested `:query` option of `:search_query` option in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/117493.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117493.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#terms_query/2","title":"ExAliyunOts.Search.terms_query/2","type":"function"},{"doc":"import MyApp.TableStore\n    search \"table\", \"index_name\",\n      search_query: [\n        query: terms_query(\"age\", [28, 29, 30])\n      ]","ref":"ExAliyunOts.Search.html#terms_query/2-example","title":"Example - ExAliyunOts.Search.terms_query/2","type":"function"},{"doc":"Use WildcardQuery as the nested `:query` option of `:search_query` option in `ExAliyunOts.search/4`.\n\nOfficial document in [Chinese](https://help.aliyun.com/document_detail/117497.html){:target=\"_blank\"} | [English](https://www.alibabacloud.com/help/en/doc-detail/117497.html){:target=\"_blank\"}","ref":"ExAliyunOts.Search.html#wildcard_query/2","title":"ExAliyunOts.Search.wildcard_query/2","type":"function"},{"doc":"import MyApp.TableStore\n\n    search \"table\", \"index_name\",\n      search_query: [\n        query: wildcard_query(\"name\", \"n*\")\n      ]","ref":"ExAliyunOts.Search.html#wildcard_query/2-example","title":"Example - ExAliyunOts.Search.wildcard_query/2","type":"function"},{"doc":"","ref":"ExAliyunOts.Search.html#t:aggregation_name/0","title":"ExAliyunOts.Search.aggregation_name/0","type":"type"},{"doc":"","ref":"ExAliyunOts.Search.html#t:field_name/0","title":"ExAliyunOts.Search.field_name/0","type":"type"},{"doc":"","ref":"ExAliyunOts.Search.html#t:group_name/0","title":"ExAliyunOts.Search.group_name/0","type":"type"},{"doc":"","ref":"ExAliyunOts.Search.html#t:options/0","title":"ExAliyunOts.Search.options/0","type":"type"},{"doc":"","ref":"ExAliyunOts.Search.html#t:order/0","title":"ExAliyunOts.Search.order/0","type":"type"},{"doc":"","ref":"ExAliyunOts.Sequence.html","title":"ExAliyunOts.Sequence","type":"module"},{"doc":"","ref":"ExAliyunOts.Sequence.html#create/2","title":"ExAliyunOts.Sequence.create/2","type":"function"},{"doc":"","ref":"ExAliyunOts.Sequence.html#delete/2","title":"ExAliyunOts.Sequence.delete/2","type":"function"},{"doc":"","ref":"ExAliyunOts.Sequence.html#delete_event/3","title":"ExAliyunOts.Sequence.delete_event/3","type":"function"},{"doc":"","ref":"ExAliyunOts.Sequence.html#next_value/2","title":"ExAliyunOts.Sequence.next_value/2","type":"function"},{"doc":"Tablestore Timeline model implements.","ref":"ExAliyunOts.Timeline.html","title":"ExAliyunOts.Timeline","type":"module"},{"doc":"","ref":"ExAliyunOts.Timeline.html#add_field/3","title":"ExAliyunOts.Timeline.add_field/3","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.html#batch_store/1","title":"ExAliyunOts.Timeline.batch_store/1","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.html#change_identifier/2","title":"ExAliyunOts.Timeline.change_identifier/2","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.html#change_seq_id/3","title":"ExAliyunOts.Timeline.change_seq_id/3","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.html#create/1","title":"ExAliyunOts.Timeline.create/1","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.html#default_seq_id_col_name/0","title":"ExAliyunOts.Timeline.default_seq_id_col_name/0","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.html#delete/2","title":"ExAliyunOts.Timeline.delete/2","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.html#drop/1","title":"ExAliyunOts.Timeline.drop/1","type":"function"},{"doc":"For manual generation sequence_id.","ref":"ExAliyunOts.Timeline.html#generate_sequence_id/0","title":"ExAliyunOts.Timeline.generate_sequence_id/0","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.html#get/3","title":"ExAliyunOts.Timeline.get/3","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.html#new/1","title":"ExAliyunOts.Timeline.new/1","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.html#scan_backward/4","title":"ExAliyunOts.Timeline.scan_backward/4","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.html#scan_forward/4","title":"ExAliyunOts.Timeline.scan_forward/4","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.html#store/2","title":"ExAliyunOts.Timeline.store/2","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.html#update/2","title":"ExAliyunOts.Timeline.update/2","type":"function"},{"doc":"Tablestore Timeline meta implements.","ref":"ExAliyunOts.Timeline.Meta.html","title":"ExAliyunOts.Timeline.Meta","type":"module"},{"doc":"","ref":"ExAliyunOts.Timeline.Meta.html#add_field/3","title":"ExAliyunOts.Timeline.Meta.add_field/3","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.Meta.html#change_identifier/2","title":"ExAliyunOts.Timeline.Meta.change_identifier/2","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.Meta.html#change_info/2","title":"ExAliyunOts.Timeline.Meta.change_info/2","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.Meta.html#create/1","title":"ExAliyunOts.Timeline.Meta.create/1","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.Meta.html#delete/1","title":"ExAliyunOts.Timeline.Meta.delete/1","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.Meta.html#drop/1","title":"ExAliyunOts.Timeline.Meta.drop/1","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.Meta.html#insert/1","title":"ExAliyunOts.Timeline.Meta.insert/1","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.Meta.html#new/1","title":"ExAliyunOts.Timeline.Meta.new/1","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.Meta.html#read/2","title":"ExAliyunOts.Timeline.Meta.read/2","type":"function"},{"doc":"","ref":"ExAliyunOts.Timeline.Meta.html#update/1","title":"ExAliyunOts.Timeline.Meta.update/1","type":"function"},{"doc":"The primary entry to use the tunnel service.\n\nThis bases on a `GenServer` module and runs under a `DynamicSupervisor` to maintain the completed\nlife cycle of the heartbeat connection, it is charge of forward the received data records to the outside\nsubscriber.\n\nHere is an example for reference:\n\n    defmodule Sync do\n      use GenServer\n\n      def start(instance, tunnel_id) do\n        GenServer.start(__MODULE__, %{instance: instance, tunnel_id: tunnel_id})\n      end\n\n      def listen(pid) do\n        GenServer.cast(pid, :listen)\n      end\n\n      @impl true\n      def init(config) do\n        {:ok, config}\n      end\n\n      @impl true\n      def handle_cast(:listen, state) do\n        ExAliyunOts.Tunnel.Worker.start_connect(state.instance, [tunnel_id: state.tunnel_id])\n        {:noreply, state}\n      end\n\n      @impl true\n      def handle_call({:record_event, {records, _next_token}}, _from, state) do\n        # ...\n        # data records changes will be received in here,\n        # and handle them in the `continue` processing,\n        # please inspect `records` for details.\n        # ...\n        {:reply, :ok, state, {:continue, records}}\n      end\n\n      @impl true\n      def handle_continue([record | reset] = records, state) do\n        # process `records` in here\n        {:noreply, state, {:continue, reset}}\n      end\n      def handle_continue([], state) do\n        {:noreply, state}\n      end\n    end\n\nHere we define a module named `Sync`, and use it like this:\n\n    instance = :my_instance\n    tunnel_id = \"...\"\n    {:ok, pid} = Sync.start(instance, tunnel_id)\n    Sync.listen(pid)\n    Process.sleep(:infinity)\n\nOr we can use this `Sync` module with other process/supervisor.","ref":"ExAliyunOts.Tunnel.Worker.html","title":"ExAliyunOts.Tunnel.Worker","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExAliyunOts.Tunnel.Worker.html#child_spec/1","title":"ExAliyunOts.Tunnel.Worker.child_spec/1","type":"function"},{"doc":"","ref":"ExAliyunOts.Tunnel.Worker.html#init/1","title":"ExAliyunOts.Tunnel.Worker.init/1","type":"function"},{"doc":"Start a supervised tunnel worker which maintains a heartbeat connection to the tunnel service internally,\nit will make the current process who calls this function as a subscriber, the subscriber will receive\ndata records status information via `handle_call/3` in `{:record_event, {records, next_token}}` message format\nlater once there are some data records changed with insert/update/delete operations, please inspect `records`\nfor details.","ref":"ExAliyunOts.Tunnel.Worker.html#start_connect/2","title":"ExAliyunOts.Tunnel.Worker.start_connect/2","type":"function"},{"doc":"* `:tunnel_id`, required, the tunnel id to setup the tunnel working flow.\n  * `:heartbeat_interval`, time to run heartbeat internally, defaults to 30 seconds, at least 5 seconds. \n  * `:heartbeat_timeout`, heartbeat timeout, defaults to 300 seconds.\n  * `:connect_timeout`, timeout setting in \"ClientConfig\" tunnel proto file, defaults to 300 seconds.\n  * `:client_tag`, optional, the custom client tag that is used to generate a tunnel client id, can customize\n  this parameter to uniquely identify tunnel workers.","ref":"ExAliyunOts.Tunnel.Worker.html#start_connect/2-options","title":"Options - ExAliyunOts.Tunnel.Worker.start_connect/2","type":"function"},{"doc":"Used for the supervisor.","ref":"ExAliyunOts.Tunnel.Worker.html#start_link/1","title":"ExAliyunOts.Tunnel.Worker.start_link/1","type":"function"},{"doc":"Stop the tunnel worker process by tunnel id.","ref":"ExAliyunOts.Tunnel.Worker.html#stop/1","title":"ExAliyunOts.Tunnel.Worker.stop/1","type":"function"},{"doc":"Common tools.","ref":"ExAliyunOts.Utils.html","title":"ExAliyunOts.Utils","type":"module"},{"doc":"","ref":"ExAliyunOts.Utils.html#attrs_to_row/1","title":"ExAliyunOts.Utils.attrs_to_row/1","type":"function"},{"doc":"","ref":"ExAliyunOts.Utils.html#downcase_atom/1","title":"ExAliyunOts.Utils.downcase_atom/1","type":"function"},{"doc":"","ref":"ExAliyunOts.Utils.html#row_to_map/1","title":"ExAliyunOts.Utils.row_to_map/1","type":"function"},{"doc":"","ref":"ExAliyunOts.Utils.html#valid_geo_point?/1","title":"ExAliyunOts.Utils.valid_geo_point?/1","type":"function"},{"doc":"# ExAliyunOts\n\n[![Coverage Status](https://coveralls.io/repos/github/xinz/ex_aliyun_ots/badge.svg?branch=master)](https://coveralls.io/github/xinz/ex_aliyun_ots?branch=master)\n[![Module Version](https://img.shields.io/hexpm/v/ex_aliyun_ots.svg)](https://hex.pm/packages/ex_aliyun_ots)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/ex_aliyun_ots/)\n[![Total Download](https://img.shields.io/hexpm/dt/ex_aliyun_ots.svg)](https://hex.pm/packages/ex_aliyun_ots)\n[![License](https://img.shields.io/hexpm/l/ex_aliyun_ots.svg)](https://github.com/xinz/ex_aliyun_ots/blob/master/LICENSE)\n[![Last Updated](https://img.shields.io/github/last-commit/xinz/ex_aliyun_ots.svg)](https://github.com/xinz/ex_aliyun_ots/commits/master)\n\nAliyun [TableStore](https://www.alibabacloud.com/product/table-store) SDK for Elixir/Erlang\n\n> TableStore is a NoSQL database service built on Alibaba Cloud’s Apsara distributed operating system that can store and access large volumes of structured data in real time.","ref":"readme.html","title":"ExAliyunOts","type":"extras"},{"doc":"```elixir\ndef deps do\n  [\n    {:ex_aliyun_ots, \"~> 0.11\"}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - ExAliyunOts","type":"extras"},{"doc":"Add these settings below into the `config/ots.secret.exs` file.\n\n```elixir\nconfig :ex_aliyun_ots, :my_instance,\n  name: \"MyInstanceName\",\n  endpoint: \"MyInstanceEndpoint\",\n  access_key_id: \"MyAliyunRAMKeyID\",\n  access_key_secret: \"MyAliyunRAMKeySecret\",\n  pool_size: 100,\n  pool_count: 1\n\nconfig :ex_aliyun_ots,\n  instances: [:my_instance],\n  debug: false,\n  enable_tunnel: false\n```\n\n* `debug`, optional, specifies whether to enable debug logger, by default it is false, and please DO NOT use debug mode in production.\n* `enable_tunnel`, optional, specifies whether to enable tunnel functions, there will startup tunnel related supervisor and registry when enable it, by default it is false.\n* `pool_size`, optional, number of connections to maintain in each pool, involved when use `Finch` as Tesla http adapter, see `Finch` [module usage](https://hexdocs.pm/finch/Finch.html#module-usage) for details, defaults to 100.\n* `pool_count`, optional, number of pools to start, involved when use `Finch` as Tesla http adapter, see `Finch` [module usage](https://hexdocs.pm/finch/Finch.html#module-usage) for details, defaults to 1.","ref":"readme.html#configuration","title":"Configuration - ExAliyunOts","type":"extras"},{"doc":"To use `ExAliyunOts`, a module that calls `use ExAliyunOts` has to be defined:\n\n```elixir\ndefmodule MyApp.TableStore do\n  use ExAliyunOts, instance: :my_instance\nend\n```\n\nThis automatically defines some macros and functions in the `MyApp.TableStore` module, here are some examples:\n\n```elixir\nimport MyApp.TableStore\n\n# Create table\ncreate_table \"table\",\n  [{\"pk1\", :integer}, {\"pk2\", :string}]\n\n# Put row\nput_row \"table\",\n  [{\"pk1\", \"id1\"}],\n  [{\"attr1\", 10}, {\"attr2\", \"attr2_value\"}],\n  condition: condition(:expect_not_exist),\n  return_type: :pk\n\n# Search index\nsearch \"table\", \"index_name\",\n  search_query: [\n  query: match_query(\"age\", 28),\n  sort: [\n    field_sort(\"age\", order: :desc)\n  ]\n]\n\n# Local transaction\nstart_local_transaction \"table\", {\"partition_key\", \"partition_value\"}\n```","ref":"readme.html#using-exaliyunots","title":"Using ExAliyunOts - ExAliyunOts","type":"extras"},{"doc":"There are two ways to use ExAliyunOts:\n\n* using macros and functions from your own ExAliyunOts module, like `MyApp.TableStore`.\n* using macros and functions from the `ExAliyunOts` module.\n\nAll defined functions and macros in `ExAliyunOts` are available and referable for your own ExAliyunOts module as well, except that the given arity of functions may different, because the `instance` parameter of each invoke request is NOT needed from your own ExAliyunOts module although the `ExAliyunOts` module defines it.","ref":"readme.html#exaliyunots-api","title":"ExAliyunOts API - ExAliyunOts","type":"extras"},{"doc":"Here is a [Tablestore adapter for Ecto](https://hex.pm/packages/ecto_tablestore) implementation as an alternative.","ref":"readme.html#ecto-adapter","title":"Ecto Adapter - ExAliyunOts","type":"extras"},{"doc":"* Table Operations\n* Row Operations\n* Conditional update\n* Auto-increment function of the primary key column\n* Filter\n* Atomic counters\n* Search index\n* Local transaction\n* Tunnel service\n* Timeline mode","ref":"readme.html#supported-functions","title":"Supported Functions - ExAliyunOts","type":"extras"},{"doc":"* Table\n  * CreateTable\n  * ListTable\n  * DeleteTable\n  * UpdateTable\n  * DescribeTable\n  * ComputeSplitPointsBySize\n\n* Row\n  * PutRow\n  * GetRow\n  * UpdateRow\n  * DeleteRow\n  * GetRange\n  * BatchGetRow\n  * BatchWriteRow\n\n* SearchIndex\n  * CreateSearchIndex\n  * DescribeSearchIndex\n  * DeleteSearchIndex\n  * Search\n  * ParallelScan\n\n* LocalTransaction\n  * StartLocalTransaction\n  * CommitTransaction\n  * AbortTransaction\n\n* SDK built-in\n  * IterateAllRange\n  * IterateParallelScan\n  * IterateSearch\n  * Sequence\n  * StreamParallelScan\n  * StreamRange\n  * StreamSearch\n\n* Tunnel service\n* Timeline mode","ref":"readme.html#supported-api","title":"Supported API - ExAliyunOts","type":"extras"},{"doc":"Thanks very much for the help and support of the Alibaba TableStore offical support team.","ref":"readme.html#thanks","title":"Thanks - ExAliyunOts","type":"extras"},{"doc":"Alibaba Tablestore product official references:\n\n* [English document](https://www.alibabacloud.com/help/doc-detail/27280.htm)\n* [中文文档](https://help.aliyun.com/document_detail/27280.html)","ref":"readme.html#references","title":"References - ExAliyunOts","type":"extras"},{"doc":"This project is licensed under the MIT license. Copyright (c) 2018- Xin Zou.","ref":"readme.html#license","title":"License - ExAliyunOts","type":"extras"},{"doc":"# Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"* Add create and delete operations to the predefined columns ([#52](https://github.com/xinz/ex_aliyun_ots/issues/52)).","ref":"changelog.html#v0-15-2-2024-03-07","title":"v0.15.2 (2024-03-07) - Changelog","type":"extras"},{"doc":"* Fix compile error when use `filter` ([#49](https://github.com/xinz/ex_aliyun_ots/pull/49)).","ref":"changelog.html#v0-15-1-2024-01-24","title":"v0.15.1 (2024-01-24) - Changelog","type":"extras"},{"doc":"* Add ValueTransferRule of SingleColumnValueFilter implemention ([#38](https://github.com/xinz/ex_aliyun_ots/pull/38)).\n* Enhance SearchIndex implementions about virtual field, group by histogram, aggregation with percentiles/top rows ([#39](https://github.com/xinz/ex_aliyun_ots/pull/39)).\n* Fix score sort of SearchIndex in runtime error ([#40](https://github.com/xinz/ex_aliyun_ots/pull/40)).\n* Add SQL query implemention ([#41](https://github.com/xinz/ex_aliyun_ots/pull/41)).\n* Enhance to simplify `condition` use ([#43](https://github.com/xinz/ex_aliyun_ots/pull/43)).\n* Fix to decode tunnel record `UPDATE_ROW` action with delete/delete_all attribute cols ([#44](https://github.com/xinz/ex_aliyun_ots/pull/44), [#47](https://github.com/xinz/ex_aliyun_ots/pull/47)).\n* Fix to properly stop tunnel worker ([#45](https://github.com/xinz/ex_aliyun_ots/pull/45)).\n* Add local transaction setting when create table. ([#46](https://github.com/xinz/ex_aliyun_ots/pull/46)).","ref":"changelog.html#v0-15-0-2024-01-22","title":"v0.15.0 (2024-01-22) - Changelog","type":"extras"},{"doc":"* Fix compile error with Elixir 1.13.0","ref":"changelog.html#v0-14-4-2021-12-14","title":"v0.14.4 (2021-12-14) - Changelog","type":"extras"},{"doc":"* Simplify dependency by removing `:timex`.","ref":"changelog.html#v0-14-3-2021-10-14","title":"v0.14.3 (2021-10-14) - Changelog","type":"extras"},{"doc":"* Implement `analyzer_parameter` when use search index.\n* Remove uesless logger when generate next proper value in sequence.","ref":"changelog.html#v0-14-2-2021-07-22","title":"v0.14.2 (2021-07-22) - Changelog","type":"extras"},{"doc":"* Add `ExAliyunOts.Search.range_query/1` to provide an intuitive experssion for use.\n* Update `ExAliyunOts.Search.range_query/2` to support use a range (in first..last format).\n* Simplify the internal config module.\n* Fix compile deprecation warning about Bitwse ^^^ in Elixir 1.12.0 version.","ref":"changelog.html#v0-14-1-2021-05-20","title":"v0.14.1 (2021-05-20) - Changelog","type":"extras"},{"doc":"* Make tunnel service usage sample and document more clear, see `ExAliyunOts.Tunnel.Worker`.\n* Rename `ExAliyunOts.Tunnel.Worker`'s `start/2` function as `start_connect/2`,\n  this could resolve potential obscurity with GenServer function naming.","ref":"changelog.html#v0-14-0-2021-04-22","title":"v0.14.0 (2021-04-22) - Changelog","type":"extras"},{"doc":"* Imporve further plainbuffer encoding(cover `crc_int32` and `crc_int64`) performance","ref":"changelog.html#v0-13-1-2021-04-20","title":"v0.13.1 (2021-04-20) - Changelog","type":"extras"},{"doc":"* Enhance CRC module to imporve plainbuffer encoding performance","ref":"changelog.html#v0-13-0-2021-04-20","title":"v0.13.0 (2021-04-20) - Changelog","type":"extras"},{"doc":"* Add `:is_atomic` option to `ExAliyunOts.batch_write/3`\n* Tweak document of `ExAliyunOts` module","ref":"changelog.html#v0-12-4-2021-04-12","title":"v0.12.4 (2021-04-12) - Changelog","type":"extras"},{"doc":"* Make compatible to use :crypto mac","ref":"changelog.html#v0-12-3-2021-03-02","title":"v0.12.3 (2021-03-02) - Changelog","type":"extras"},{"doc":"* Add `ExAliyunOts.stream_search/4`\n* Add `ExAliyunOts.iterate_search/4`\n* Fix compile warning when generate docs","ref":"changelog.html#v0-12-2-2021-01-21","title":"v0.12.2 (2021-01-21) - Changelog","type":"extras"},{"doc":"* Fix Elixir 1.11 compilation warnings, explicitly add `Jason` into mix.exs deps","ref":"changelog.html#v0-12-1-2021-01-15","title":"v0.12.1 (2021-01-15) - Changelog","type":"extras"},{"doc":"* Fix Elixir 1.11 compilation warnings","ref":"changelog.html#v0-12-0-2021-01-15","title":"v0.12.0 (2021-01-15) - Changelog","type":"extras"},{"doc":"* Remove internal error logger print when occurs `OTSConditionCheckFail`, `OTSObjectAlreadyExist` and `OTSObjectNotExist`","ref":"changelog.html#v0-11-1-2021-01-12","title":"v0.11.1 (2021-01-12) - Changelog","type":"extras"},{"doc":"**NOTICE**: Since this version changes, when upgrade this library to `0.11.0` or higher version, if also use\n`ecto_tablestore`, please upgrade `ecto_tablestore` to `0.8.0` or higher version.\n\n* Use `protox` to replace `exportobuf` for google protobuf library, and maintain the generated modules in [hexpm](https://hex.pm/packages/tablestore_protos)\n* Add retry for http request timeout\n* Some implicit modules are removed and directly use the generated protobuf modules, cover:\n    - Use ExAliyunOts.TableStoreFilter.Filter(in `tablestore_protos`) to replace `ExAliyunOts.Var.Filter`\n    - Use ExAliyunOts.TableStoreFilter.SingleColumnValueFilter(in `tablestore_protos`) to replace `ExAliyunOts.Var.SingleColumnValueFilter`\n    - Use ExAliyunOts.TableStoreFilter.ColumnPaginationFilter(in `tablestore_protos`) to replace `ExAliyunOts.Var.ColumnPaginationFilter`\n    - Use ExAliyunOts.TableStore.Condition(in `tablestore_protos`) to replace `ExAliyunOts.Var.Condition`\n* Improve module file struct/naming\n* Add `ExAliyunOts.create_index/6` to create global secondary indexes\n* Add `index_metas` and `defined_columns` optional options when use `ExAliyunOts.create_table/4`","ref":"changelog.html#v0-11-0-2020-12-10","title":"v0.11.0 (2020-12-10) - Changelog","type":"extras"},{"doc":"* Improve plainbuffer decoding performance","ref":"changelog.html#v0-10-0-2020-12-04","title":"v0.10.0 (2020-12-04) - Changelog","type":"extras"},{"doc":"* [Http] Use Tesla with Finch adapter","ref":"changelog.html#v0-9-0-2020-11-17","title":"v0.9.0 (2020-11-17) - Changelog","type":"extras"},{"doc":"* [Table] Add `ExAliyunOts.compute_split_points_by_size/3`\n* [Table] Fix `shard_splits` of DescribeTable response in proper decoded format\n* [SearchIndex] Add ParallelScan functions\n\t- Implement standard api `ExAliyunOts.compute_splits/3`\n\t- Implement standard api `ExAliyunOts.parallel_scan/4`\n\t- Implement built-in functions `ExAliyunOts.iterate_parallel_scan/5`, `ExAliyunOts.iterate_parallel_scan/7` and\n    `ExAliyunOts.stream_parallel_scan/4` to make general use cases simple","ref":"changelog.html#v0-8-0-2020-11-13","title":"v0.8.0 (2020-11-13) - Changelog","type":"extras"},{"doc":"* Enhance condition expression and simplify some code about it","ref":"changelog.html#v0-7-1-2020-09-15","title":"v0.7.1 (2020-09-15) - Changelog","type":"extras"},{"doc":"* Update `ExAliyunOts.filter/1` for a better usage (please notice that this change is incompatible update),\n  before this version uses `filter` like this:\n\n  ```\n  filter(\n    \"name[ignore_if_missing: true, latest_version_only: true]\" == var_name and\n      \"age\" > 1\n  )\n  ```\n\n  after this version uses `filter` like this, and then the column_name \"name\" can bind variable of the context\n\n  ```\n  filter(\n    {\"name\", ignore_if_missing: true, latest_version_only: true} == var_name and\n      \"age\" > 1\n  )\n  ```","ref":"changelog.html#v0-7-0-2020-09-15","title":"v0.7.0 (2020-09-15) - Changelog","type":"extras"},{"doc":"* Add `ExAliyunOts.stream_range/5`\n* Add index_sorts option when create search index\n* Remove Mix in runtime\n* Support filter expression column_name to bind variable of the context\n* Enhance condition expression\n* Tweak plainbuffer\n* Downgrade hackney to 1.15.2","ref":"changelog.html#v0-6-10-2020-09-15","title":"v0.6.10 (2020-09-15) - Changelog","type":"extras"}]}